#!/usr/bin/env python
#
# 
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/write_component_energies.py,v 1.4 2011/01/10 23:18:41 rhuey Exp $
#
import os 

from MolKit import Read
from PyAutoDock.InternalEnergy import InternalEnergy
from PyAutoDock.MolecularSystem import MolecularSystem
from PyAutoDock.AutoDockScorer import AutoDock4Scorer, AutoDock41Scorer




if __name__ == '__main__':
    import sys
    import getopt


# write_component_energies.py -r receptorfilename -l ligandfilename [-o outputfilename -a append_to_file] -v
    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: write_component_energies.py -r receptorfilename -l ligandfilename"
        print
        print "    Description of command..."
        print "         -r     receptorfilename"
        print "         -l     ligandfilename"
        print "    Optional parameters:"
        print "        [-o]    outputfilename (default is 'component_energies.txt')"
        print "        [-w]    overwrite_file (default mode is to append)"
        print "        [-n]    use AutoDock4 weights (default mode uses AutoDock41 weights)"
        print "        [-v]    verbose output"

    # initialize required parameters
    #-r: receptorfilename
    receptorfilename =  None
    #-l: ligandfilename
    ligandfilename =  None
    #-a append to outputfile
    append_to_outputfile = True
    # optional parameters
    verbose = None
    #-o outputfilename
    outputfilename = 'component_energies.txt'
    #-n use_autodock40_weights 
    use_autodock40_weights = False

    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'r:l:o:anvh')
    except getopt.GetoptError, msg:
        print 'write_component_energies.py: %s' %msg
        usage()
        sys.exit(2)

    #'r:l:o:wnvh'
    for o, a in opt_list:
        #print "o=", o, " a=", a
        if o in ('-r', '--r'):
            receptorfilename = a
            if verbose: print 'set receptorfilename to ', a
        if o in ('-l', '--l'):
            ligandfilename = a
            if verbose: print 'set ligandfilename to ', a
        if o in ('-o', '--o'):
            outputfilename = a
            if verbose: print 'set outputfilename to ', a
        if o in ('-w', '--w'):
            append_to_outputfile = False
            if verbose: print 'set append_to_outputfile to ', append_to_outputfile
        if o in ('-n', '--n'):
            use_autodock40_weights = True
            if verbose: print 'set use_autodock41_weights to ', use_autodock41_weights
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to ', True
        if o in ('-h', '--'):
            usage()
            sys.exit()


    if not receptorfilename:
        print 'write_component_energies: receptorfilename must be specified.'
        usage()
        sys.exit()

    if not ligandfilename:
        print 'write_component_energies: ligandfilename must be specified.'
        usage()
        sys.exit()

    receptor = Read(receptorfilename)[0]
    if verbose: print 'read ', receptorfilename
    receptor.buildBondsByDistance()

    ligand = Read(ligandfilename)[0]
    if verbose: print 'read ', ligandfilename
    ligand.buildBondsByDistance()
    mode = 'w'
    first = True
    if append_to_outputfile:
        mode = 'a'
        first = not os.path.exists(outputfilename)
    if verbose: print 'first is ', first
    optr = open(outputfilename, mode)
    if first:
        tstr = "RECEPTOR_LIGAND         #ESTAT       #HB      #VDW    #DSOLV  #TORSDOF #ATS #TORS\n"
        optr.write(tstr)
    #get the scores
    ms = MolecularSystem()
    ms.add_entities(receptor.allAtoms)
    ms.add_entities(ligand.allAtoms)
    if use_autodock40_weights:
        ad_scorer = AutoDock4Scorer()
    else:
        ad_scorer = AutoDock41Scorer()
    ad_scorer.set_molecular_system(ms)
    score_list = ad_scorer.get_score_per_term()
    ostr = "%8s_%6s," %(receptor.name,ligand.name)
    for score in score_list:
        ostr = ostr + " % 8.4f," %score
    ostr = ostr + " % 8d," %ligand.TORSDOF
    ostr = ostr + " % 8d," %len(ligand.allAtoms)
    ostr = ostr + " % 8d," %len(ligand.torTree.torsionMap)
    ostr = ostr + "\n"   
    optr.write(ostr)
    optr.close()
    if verbose:
        print 'wrote %s' %outputfilename


# To execute this command type:
# write_component_energies.py -r receptorfilename -l ligandfilename [-o outputfilename -a append_to_file -n use_autodock40_weights] -v







#!/usr/bin/env python
#
# 
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/summarize_results_vif.py,v 1.1 2012/01/24 23:57:09 rhuey Exp $
#
# $Id: summarize_results_vif.py,v 1.1 2012/01/24 23:57:09 rhuey Exp $ 
#
import os, glob

from MolKit import Read
from MolKit.hydrogenBondBuilder import HydrogenBondBuilder
from AutoDockTools.Docking import Docking
from mglutil.math.rmsd import RMSDCalculator
from PyAutoDock.InternalEnergy import InternalEnergy
from PyAutoDock.MolecularSystem import MolecularSystem
from PyAutoDock.AutoDockScorer import AutoDock4Scorer
from numpy import oldnumeric as Numeric


num_atoms = 0
num_hydrogens = 0
num_heavy_atoms = 0
be = 0.
be_lc = 0.
#vif
lp_ats = None
ls_ats = None
rp_ats = None
lp_energy = 0
ls_energy = 0
rp_energy = 0
report_energy_breakdown = False

def get_filename(d, conf):
    #find the filename of dlg containing this conformation
    for k in d.dlo_list:
        if conf in k.conformations:
            dlo_filename = os.path.splitext(k.parser.filename)[0]
            break
    return dlo_filename


def get_energy_breakdown(receptor_atoms, ligand_atoms, coords_to_use):
    ms = MolecularSystem()
    ms.add_entities(receptor_atoms)
    ms.add_entities(ligand_atoms)
    ms.set_coords(1, coords_to_use) 
    #ms.set_coords(1, c.getCoords()[:])
    ad_scorer = AutoDock4Scorer()
    ad_scorer.set_molecular_system(ms)
    score_list = ad_scorer.get_score_per_term()
    ostr = ""
    for score in score_list:
        ostr = ostr + "% 8.4f," %score
    return ostr[:-1]


def construct_hydrogen_bonds(receptor_atoms, ligand_atoms, verbose=False):
    atDict = HydrogenBondBuilder().build(receptor_atoms, ligand_atoms)
    num_hbonds = len(atDict)
    if verbose:
        print "found ", num_hbonds, " hydrogen bonds"
        for k,v in atDict.items():
            print  k.full_name(), ' ',len(v), " hydrogen bond(s) to ",
            for a in atDict[k]:
                print a.full_name(), " ",
            print
    ostr = "%2d," % num_hbonds
    return ostr


def get_best_energy_info(d, rms_tolerance, build_hydrogen_bonds=False, \
                                report_energy_breakdown=False,
                                receptor_filename=None, refCoords=None,
                                subtract_internal_energy=False):
    global be
    ostr = "" 
    clust0 = d.clusterer.clustering_dict[rms_tolerance][0]
    c = clust0[0]
    d.ch.set_conformation(c)
    #find the filename of the best result
    dlo_filename = get_filename(d, c)
    #print "83: set dlo_filename to ", dlo_filename
    if verbose: print "set dlo_filename to ", dlo_filename
    ostr = dlo_filename + ", "
    be = c.binding_energy
    #print "87: set be to ", be
    if subtract_internal_energy:
        be = c.binding_energy - c.total_internal
        #print "90: after subtracting_internal_energy be = ", be
    if refCoords:
        ostr += "%3d,%3d,%3d,% 8.4f,% 8.4f," %(len(d.clusterer.data), num_clusters, len(clust0),  be, c.getRMSD(refCoords=refCoords))
        #print "93: len(clust0)=", len(clust0), ", be=", be, " rmsd_LE=", c.getRMSD(refCoords=refCoords)
        #print "94: ostr=", ostr
        #ostr += "%3d,%3d,%3d,% 8.4f,% 8.4f," %(len(d.clusterer.data), num_clusters, len(clust0),  c.binding_energy, c.getRMSD(refCoords=refCoords))
    else:
        ostr += "%3d,%3d,%3d,% 8.4f,% 8.4f," %(len(d.clusterer.data), num_clusters, len(clust0),  be, c.getRMSD())
        print "99:  rmsd_LE=", c.getRMSD(), " @@@"
        #print "98: len(clust0)=", len(clust0), ", be=", be, " rmsd_LE=", c.getRMSD()
        #print "99: ostr=", ostr
        #ostr += "%3d,%3d,%3d,% 8.4f,% 8.4f," %(len(d.clusterer.data), num_clusters, len(clust0),  c.binding_energy, c.getRMSD())
    receptor = None
    if build_hydrogen_bonds:
        if not receptor_filename:
            print "receptor_filename must be specified in order to build_hydrogen_bonds"
            return 
        receptor = Read(receptor_filename)[0] 
        receptor.buildBondsByDistance()
        d.ligMol.buildBondsByDistance()
        ostr += construct_hydrogen_bonds(receptor.allAtoms, d.ligMol.allAtoms, verbose=verbose)
    if report_energy_breakdown:
        if not receptor_filename:
            print "receptor_filename must be specified in order to build_hydrogen_bonds"
        if not receptor:
            receptor = Read(receptor_filename)[0] 
            receptor.buildBondsByDistance()
        #build bonds if necessary 
        d.ligMol.buildBondsByDistance()
        #ostr += get_energy_breakdown(receptor.allAtoms, d.ligMol.allAtoms, c)
        ostr += get_energy_breakdown(receptor.allAtoms, d.ligMol.allAtoms, c.getCoords()[:])
    print "128: len(lp_ats)=", len(lp_ats)
    if len(lp_ats):
        try:
            ostr += " % 8.4f," % Numeric.add.reduce(lp_ats.estat_energy+lp_ats.vdw_energy)
            print "131: lp_ats 1: ostr=", ostr, "\n"
        except:
            ostr += "         " 
    if len(ls_ats):
        ostr += " % 8.4f," % Numeric.add.reduce(ls_ats.estat_energy+ls_ats.vdw_energy)
        print "135 ls_ats 2: ostr=", ostr, "\n"
    print "len(rp_ats)=", len(rp_ats)
    if len(rp_ats):
        ostr += " % 8.4f," % Numeric.add.reduce(rp_ats.estat_energy+rp_ats.vdw_energy)
        print "139: ostr=", ostr, "\n"
    return ostr


def get_largest_cluster_info(d, rms_tolerance, build_hydrogen_bonds=False, \
                                    report_energy_breakdown=False,
                                    receptor_filename=None, refCoords=None,
                                    subtract_internal_energy=False):
    global be_lc
    largest = d.clusterer.clustering_dict[rms_tolerance][0]
    if verbose: print "set largest to ", len(largest)
    for clust in d.clusterer.clustering_dict[rms_tolerance]:
        if verbose: print "current largest cluster len= ", len(clust)
        if len(clust)>len(largest): 
            if verbose: print "resetting largest clust: now len=", len(clust)
            largest = clust
    c = largest[0]   #print info about the lowest energy member of this cluster
    d.ch.set_conformation(c)
    ostr = " "
    dlo_filename = get_filename(d, c)
    print "lc: dlo_filename= ", dlo_filename
    ostr += dlo_filename + ","
    print "161 ostr=", ostr, "\n"
    be_lc = c.binding_energy
    print "163 set be_lc to ", be_lc, "\n"
    if subtract_internal_energy:
        be_lc = c.binding_energy - c.total_internal
    if refCoords:
        ostr += "%3d,%3d,%3d,% 8.4f,% 8.4f," %(len(d.clusterer.data), num_clusters, len(largest),  be_lc, c.getRMSD(refCoords=refCoords))
    else:
        ostr += "%3d,%3d,%3d,% 8.4f,% 8.4f," %(len(d.clusterer.data), num_clusters, len(largest),  be_lc, c.getRMSD())
    print "170 2c: ostr=", ostr, "\n"
    if verbose: print "set dlo_filename to ", dlo_filename
    #update the coords only if you need to?
    receptor = None
    if build_hydrogen_bonds:
        if not receptor_filename:
            print "receptor_filename must be specified in order to build_hydrogen_bonds"
            return 
        receptor = Read(receptor_filename)[0] 
        receptor.buildBondsByDistance()
        ostr += construct_hydrogen_bonds(receptor.allAtoms, d.ligMol.allAtoms, verbose=verbose)
    if report_energy_breakdown:
        if not receptor_filename:
            print "receptor_filename must be specified in order to build_hydrogen_bonds"
            return 
        if not receptor:
            receptor = Read(receptor_filename)[0] 
            receptor.buildBondsByDistance()
        #ostr += get_energy_breakdown(receptor.allAtoms, d.ligMol.allAtoms, c)
        ostr += get_energy_breakdown(receptor.allAtoms, d.ligMol.allAtoms, c.getCoords()[:])
    if len(lp_ats):
        ostr += " % 8.4f," % Numeric.add.reduce(lp_ats.estat_energy+lp_ats.vdw_energy)
        print "199  lp_ats:  ", ostr, "\n"
    if len(ls_ats):
        ostr += " % 8.4f," % Numeric.add.reduce(ls_ats.estat_energy+ls_ats.vdw_energy)
        print "202 ls_ats:  ", ostr, "\n"
    if len(rp_ats):
        ostr += " % 8.4f," % Numeric.add.reduce(rp_ats.estat_energy+rp_ats.vdw_energy)
        print "205 rp_ats:  ", ostr, "\n"
    return ostr


if __name__ == '__main__':
    import sys
    import getopt


    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: summarize_results_vif.py -d directory"
        print
        print "    Description of command..."
        print "         -d     directory"
        print "    Optional parameters:"
        print "        [-t]    rmsd tolerance (default is 1.0)"
        print "        [-f]    rmsd reference filename "
        print "        (default is to use input ligand coordinates from docking log)"
        print "        [-b]    print best docking info only (default is print all)"
        print "        [-L]    print largest cluster info only (default is print all)"
        print "        [-B]    print best docking and largest cluster info only (default is print all)"
        print "        [-o]    output filename"
        print "                      (default is 'summary_of_results')"
        print "        [-a]    append to  output filename"
        print "                      (default is to open output filename 'w')"
        print "        [-k]    build hydrogen bonds and report number built"
        print "        [-e]    compute estat, vdw, hb + desolv energies and report breakdown"
        print "        [-r]    receptor filename"
        print "        [-i]    subtract internal energy"
        print "        [-p]    report depth of torsion tree"
        print "        [-v]    verbose output"


    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'd:o:t:f:r:bLBkaueipvh')
    except getopt.GetoptError, msg:
        print 'summarize_results_vif.py: %s' %msg
        usage()
        sys.exit(2)

    # initialize required parameters
    #-d: directory
    directory =  None
    #-t: rms_tolerance
    rms_tolerance =  1.0
    #-f: rms reference
    rms_reference = None
    # optional parameters
    verbose = None
    #-o outputfilename
    outputfilename = "summary_of_results"
    #-r receptor_filename
    receptor_filename = None
    #-b print best only
    print_best_only = False
    #-L print Largest cluster only
    print_Largest_only = False
    #-B print best and Largest cluster only
    print_best_and_Largest_only = False
    #-k build hydrogen bonds 
    build_hydrogen_bonds = False
    #-a append to outputfile
    append_to_outputfile = False
    #-e report_energy_breakdown report_energy_breakdown = False
    #-p report_torsion_tree_depth
    report_torsion_tree_depth = False
    #-i subtract_internal_energy
    subtract_internal_energy = False

    #'d:o:t:r:f:bLBkaueivh'
    for o, a in opt_list:
        #print "o=", o, " a=", a
        if o in ('-d', '--d'):
            directory = a
            if verbose: print 'set directory to ', a
        if o in ('-o', '--o'):
            outputfilename = a
            if verbose: print 'set outputfilename to ', a
        if o in ('-t', '--t'):
            rms_tolerance = float(a)
            if verbose: print 'set rms_tolerance to ', a
        if o in ('-f', '--f'):
            rms_reference = a
            if verbose: print 'set rms_reference to ', a
        if o in ('-r', '--r'):
            receptor_filename = a
            if verbose: print 'set receptor_filename to ', a
        if o in ('-b', '--b'):
            print_best_only = True
            if verbose: print 'set print_best_only to ', True
        if o in ('-L', '--L'):
            print_Largest_only = True
            if verbose: print 'set print_Largest_only to ', True
        if o in ('-B', '--B'):
            print_best_and_Largest_only = True
            print_best_only = True
            print_Largest_only = True
            if verbose: print 'set print_best_and_Largest_only to ', True
        if o in ('-k', '--k'):
            build_hydrogen_bonds = True
            if verbose: print 'set build_hydrogen_bonds to ', True
        if o in ('-a', '--a'):
            append_to_outputfile = True
            if verbose: print 'set append_to_outputfile to ', True
        #this option is apparently working currently:
        if o in ('-e', '--e'):
            report_energy_breakdown = True
            if verbose: print 'set report_energy_breakdown to ', True
        if o in ('-i', '--i'):
            subtract_internal_energy = True
            if verbose: print 'set subtract_internal_energy to ', True
        if o in ('-p', '--p'):
            report_torsion_tree_depth= True
            if verbose: print 'set report_torsion_tree_depth to ', True
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to ', True
        if o in ('-h', '--'):
            usage()
            sys.exit()

    if outputfilename=='summary_of_results':
        outputfilename = outputfilename + '_' + str(rms_tolerance)
    if build_hydrogen_bonds is True and receptor_filename is None:
        print 'to build hydrogen bonds, receptor filename must be specified'
        sys.exit()
    if report_energy_breakdown is True and receptor_filename is None:
        print 'to report energy breakdown, receptor filename must be specified'
        sys.exit()
    if not  directory:
        print 'summarize_results_vif: directory must be specified.'
        usage()
        sys.exit()

    #read all the docking logs in as one Docking
    dlg_list = glob.glob(directory + '/*.dlg')
    d = Docking()
    for dlg in dlg_list:
        #code contributed by S.Forli
        try:  
            d.readDlg(dlg)
        except:
            print "Error reading the file \"%s\""  % dlg
            #break
    if not d.dlo_list:
        print "Sorry, no valid DLG results found in the path : %s" % directory
        sys.exit(1)

    #when >50 truncate the dockings at 50
    #d.dlo_list = d.dlo_list[:50]
    if receptor_filename is not None:
        receptor_filename = directory + '/' + receptor_filename
    #setup rmsd tool
    coords = d.ligMol.allAtoms.coords[:]
    lp_ats = d.ligMol.allAtoms.get(lambda x: x.autodock_element=='LP')
    ls_ats = d.ligMol.allAtoms.get(lambda x: x.autodock_element=='LS')
    rp_ats = d.ligMol.allAtoms.get(lambda x: x.autodock_element=='RP')
    refCoords = None
    if rms_reference is not None:
        file = directory + '/' + rms_reference
        ref = Read(file)
        if not len(ref): 
            print "unable to read ", rms_reference
        else:
            ref = ref[0]
            coords = ref.allAtoms.coords
            refCoords = coords[:]
    d.clusterer.rmsTool = RMSDCalculator(coords)
    d.clusterer.make_clustering(rms_tolerance) 
    mode = 'w'
    first = True
    if append_to_outputfile:
        mode = 'a'
        first = not os.path.exists(outputfilename)
    fptr = open(outputfilename, mode)
    num_clusters = len(d.clusterer.clustering_dict[rms_tolerance])
    #output a header if file opened with mode 'w' or if this is first use of outputfile...
    if verbose: print "first is ", first
    if first:
        tstr = ""
        if print_best_only:
            tstr += "lowestEnergy_dlgfn #runs #cl #LEC LE LP_E RP_E LS_E rmsd_LE "  #report energy for LP,RP,LS ats
            #if build_hydrogen_bonds:
            #    tstr+= "#hb "
            #if report_energy_breakdown:
            #    tstr+= "#ESTAT #HB #VDW #DSOLV "
        if print_Largest_only:
            if print_best_only:
                tstr += "   "
            tstr += "largestCl_dlgfn #runs #cl #LEC LE LP_E RP_E LS_E rmsd_LC "  #report energy for LP,RP,LS ats
            #if build_hydrogen_bonds:
            #    tstr+= "#hb "
            #if report_energy_breakdown:
            #    tstr+= "#ESTAT #HB #VDW #DSOLV "
        if not print_best_only and not print_Largest_only:
            tstr = "#dlgfn #incluster  #LE    LP_E   RP_E   LS_E #rmsd "
            #if build_hydrogen_bonds:
            #    tstr+= "#hb "
            #if report_energy_breakdown:
            #    tstr+= "#ESTAT #HB #VDW #DSOLV "
        tstr+= "#ats #hydrogens lig_eff" 
        #tstr+= "#ats #tors "
        #if report_torsion_tree_depth:
        #    tstr+= "#depthTT "
        #tstr+= "#h_ats #lig_eff "
        tstr+="\n"
        fptr.write(tstr)
        first = False
    #set up counts
    ligAts = d.ligMol.allAtoms
    total_num_atoms = len(ligAts)
    num_hydrogen_atoms = len(ligAts.get(lambda x: x.element=='H'))
    if verbose: print "num_hydrogen_atoms = ", num_hydrogen_atoms
    num_heavy_atoms = total_num_atoms - num_hydrogen_atoms
    if verbose: print "num_heavy_atoms = ", num_heavy_atoms
    ####lp_ats = ligAts.get(lambda x: x.autodock_element=='LP')
    ####ls_ats = ligAts.get(lambda x: x.autodock_element=='LS')
    ####rp_ats = ligAts.get(lambda x: x.autodock_element=='RP')
    #torTree_depth = d.ligMol.torTree.get_depth()
    ostr = ""
    if print_best_only or print_Largest_only:
        if print_best_only:
            if verbose: print 'print_best_only is True'
            ostr += get_best_energy_info(d, rms_tolerance, build_hydrogen_bonds, report_energy_breakdown, receptor_filename, refCoords=refCoords, subtract_internal_energy=subtract_internal_energy)
        if print_Largest_only:
            if verbose: print 'print_Largest_only is True'
            if len(ostr): ostr += ','
            ostr += get_largest_cluster_info(d, rms_tolerance, build_hydrogen_bonds, report_energy_breakdown, receptor_filename, refCoords=refCoords, subtract_internal_energy=subtract_internal_energy)
        #add number of atoms and number of torsions to output string and depth of torsion tree
        #ostr += "%4d,%2d" %(len(d.ligMol.allAtoms), d.ligMol.parser.keys.count('BRANCH'))
        ostr += " %4d," %(total_num_atoms)
        if  report_torsion_tree_depth:
            ostr += " %2d," %(torTree_depth)
        lig_eff = be/num_heavy_atoms
        ostr += " %2d,% 8.4f," %(num_hydrogen_atoms, lig_eff)
        ostr +='\n'
        try:
            fptr.write(ostr)
        except:
            print 'except'
            fptr.write("\n")
    else:
        receptor = None
        if receptor_filename is not None:
            receptor = Read(receptor_filename)
            receptor = receptor[0]
            receptor.buildBondsByDistance()
        clust_ind = 0
        for clust in d.clusterer.clustering_dict[rms_tolerance]:
            #for each cluster in the clustering at this rms_tolerance:
            #output length of cluster and energies of the lowest energy result
            c = clust[0]
            d.ch.set_conformation(c)
            #write the dlg filename for the best in this cluster
            dlo_filename = get_filename(d, c)
            ostr = dlo_filename + "," 
            fptr.write(ostr)
            be = c.binding_energy
            if subtract_internal_energy:
                be = c.binding_energy - c.total_internal
            #print "477: clust_index=", clust_ind, "len(clust)=", len(clust), ", be=", be, " rmsd_LE=", c.getRMSD(refCoords=refCoords)
   
    
            #ostr = "%d,% 8.4f,% 8.4f" %(len(clust), be, c.getRMSD(refCoords=refCoords))
            ostr = "%d,% 8.4f," %(len(clust), be)
            if len(lp_ats):
                ostr += " % 8.4f," % Numeric.add.reduce(lp_ats.estat_energy+lp_ats.vdw_energy)
            else:
                ostr += " %     0.0,"
            if len(rp_ats):
                ostr += "% 8.4f," % Numeric.add.reduce(rp_ats.estat_energy+rp_ats.vdw_energy)
            else:
                ostr += " %     0.0,"
            #ostr = "%3d,% 8.4f,% 8.4f" %(len(clust), c.binding_energy, c.getRMSD(refCoords=refCoords))
            if len(ls_ats):
                ostr += " % 8.4f," % Numeric.add.reduce(ls_ats.estat_energy+ls_ats.vdw_energy)
            else:
                ostr += " %     0.0,"
            #add rmsd
            ostr += " % 8.4f," % c.getRMSD()
            ostr += " %4d," %(total_num_atoms)
            lig_eff = be/num_heavy_atoms
            #ostr += ", %2d,% 8.4f" %(num_hydrogen_atoms, lig_eff)
            ostr += " %2d, % 8.4f" %(num_hydrogen_atoms, lig_eff)
            ostr += "\n"
            fptr.write(ostr)
            clust_ind+=1
    fptr.close()
    
    

# To execute this command type:
# summarize_results_vif.py -d directory -t rmsd tolerance 
# -b report info about lowest energy docked result 
# -f rms reference filename
# -L report largest cluster information 
# -B print best docking info  and largest cluster info only 
# -r receptor filename  -k build hydrogen bonds -o outputfilename 
# -a append to  outputfilename  -e report energy breakdown"
# -i subtract internal energy
# -p report depth of torsion tree 
# -v verbose output



#
#
#
#
# $Id: test_atomTypeTools.py,v 1.1 2004/06/08 19:18:45 rhuey Exp $
#

import unittest, os
from string import split
from AutoDockTools.atomTypeTools import SolvationParameterizer
from MolKit import molecule, protein
molecule.bhtreeFlag = 0
protein.bhtreeFlag = 0
from MolKit import Read
from MolKit.pdbParser import PdbParser

from MolKit.chargeCalculator import GasteigerChargeCalculator, KollmanChargeCalculator


class SolvationParameterizerBaseTests(unittest.TestCase):
    def setUp(self):
        #read it as a pdb file to remove charges + solpars
        mols = PdbParser('hsg1.pdbqs').parse()
        self.mol = mols[0]
        self.mol.buildBondsByDistance()

        kollCC = KollmanChargeCalculator()
        kollCC.addCharges(self.mol.allAtoms)
    

    def tearDown(self):
        """
        clean-up
        """
        del(self.mol)


    def test_constructor(self):
        """
        instantiate an SolvationParameterizer
        """
        solpar = SolvationParameterizer()
        self.assertEquals(solpar.__class__, SolvationParameterizer)



    def test_addParameters(self):
        """
         test that SolvationParameterizer can addParameters
        """
        solpar = SolvationParameterizer()
        notfound = solpar.addParameters(self.mol.allAtoms)
        self.assertEquals(len(notfound), 0)


    def test_addParameters_vs_addsol(self):
        """
         test that SolvationParameterizer correctly handles charges
        """
        solpar = SolvationParameterizer()
        notfound = solpar.addParameters(self.mol.allAtoms)
        mol2 = Read("hsg1.pdbqs")[0]
        self.mol.allAtoms.sort()
        mol2.allAtoms.sort()
        for a1,a2 in zip(self.mol.allAtoms, mol2.allAtoms):
            self.assertEquals(a1.AtVol, a2.AtVol)
            self.assertEquals(a1.AtSolPar, a2.AtSolPar)



if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# 
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/process_VSResults.py,v 1.17 2011/03/29 17:36:21 rhuey Exp $
#
# $Id: process_VSResults.py,v 1.17 2011/03/29 17:36:21 rhuey Exp $
#
import os, glob, time

from MolKit import Read
from AutoDockTools.Docking import Docking, DockingResultProcessor
from mglutil.math.rmsd import RMSDCalculator
from AutoDockTools.InteractionDetector import InteractionDetector
from MolKit.hydrogenBondBuilder import HydrogenBondBuilder


def get_filename(d, conf):
    #find the filename of dlg containing this conformation
    for k in d.dlo_list:
        if conf in k.conformations:
            dlo_filename = os.path.splitext(k.parser.filename)[0]
            break
    return dlo_filename




if __name__ == '__main__':
    import sys
    import getopt

    #'d:t:f:r:BLx:Dnc vh'

    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: process_VSResults.py -d directory"
        print
        print "    Description of command..."
        print "         -d     directory"
        print "    Optional parameters:"
        print "        [-t]    rmsd tolerance (default is 2.0)"
        print "        [-f]    rmsd reference filename "
        print "        (default is to use input ligand coordinates from a docking log)"
        print "        [-r]    receptor filename (default is set from gridmap names)"
        print "        [-B]    create best docking pdbqt only (default is create both best energy and largest cluster )"
        print "        [-L]    create largest cluster docking pdbqt only (default is create both best energy and largest cluster )"
        print "        [-l]    stem for largest cluster docking pdbqt file (default is 'ligandname_lc')"
        print "        [-x]    maximum number of clusters to report (default is all)"
        print "        [-Z]    do not include interactions in output pdbqt file(default is to include interactions)"
        print "        [-n]    do not build hydrogen bonds (default is to build hydrogen bonds + report)"
        print "        [-c]    do not detect atoms in close contact (default is to detect + report)"
        print "        [-p]    include detection of pi-pi interactions in close contact (default is not to detect pi-pi and pi-cation)"
        print "        [-H]    custom HydrogenBondBuilder parameters: comma-separated-list eg distCutoff=3.00,distCutoff2=3.25"
        print "        [-v]    verbose output"


    #HydrogenBondBuilder default parameter values:
    #   distCutoff=2.25     #H-Acc Distance cutoff
    #   distCutoff2=3.00    #Donor-Acc Distance cutoff
    #   d2min=120           #sp2-donor-hydrogen-acceptor angle 
    #   d2max=180           #
    #   d3min=120           #sp3-donor-hydrogen-acceptor angle
    #   d3max=180           #
    #   a2min=110           #sp2-donor-acceptor-acceptorN angle, 
    #   a2max=150           #    where acceptorN is 'neighbor' atom bonded to acceptor
    #   a3min=100           #sp3-donor-acceptor-acceptorN angle
    #   a3max=150           #    where acceptorN is 'neighbor' atom bonded to acceptor

    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'd:t:f:r:BLl:x:DZncpH:vh')
    except getopt.GetoptError, msg:
        print 'process_VSResults.py: %s' %msg
        usage()
        sys.exit(2)
    
    # initialize required parameters
    #-d: directory
    directory =  None

    # optional parameters
    #-o: outputfilename
    outputfilename =  None
    #-t: rms_tolerance
    rms_tolerance =  2.0
    #-f: ligand rms reference
    rms_reference = None
    #-r receptor_filename 
    #   if None, set from gridmap names
    receptor_filename = None
    # default is create 2 pdbqt files
    write_both = True
    #-B best only
    best_only = False
    #-L largestCl only
    largestCl_only = False
    #-l stem for largest cluster pdbqtfile 
    lc_stem = ""
    #-x max_cl_to_write 
    #?SHOULD default be all? if so initialize to -1
    #max_cl_to_write = 10
    max_cl_to_write = -1

    #-D include_interactions in output pdbqt file
    #-Z do not include_interactions in output pdbqt file
    include_interactions = True
    #-p include detection of pi-pi and pi-cation interactions 
    detect_pi = False

    #-n do NOT build hydrogen bonds
    build_hbonds = True
    #-c do NOT detect_close_contscts
    detect_close_contacts = True

    #-H custom hydrogen bond parameters
    custom_hb_parameters = ""

    # -v verbose output
    verbose = None

    # d:t:f:r:BLl:x:DZncpH:vh

    for o, a in opt_list:
        #print "o=", o, " a=", a
        if o in ('-d', '--d'):
            directory = a
            if verbose: print 'set directory to ', a
        if o in ('-t', '--t'):
            rms_tolerance = float(a)
            if verbose: print 'set rms_tolerance to ', rms_tolerance
        if o in ('-f', '--f'):
            rms_reference = a
            if verbose: print 'set rms_reference filename to ', rms_reference
        if o in ('-r', '--r'):
            receptor_filename = a
            if verbose: print 'set receptor_filename to ', receptor_filename
        if o in ('-B', '--B'): #create pdbqt file for bestenergy result only
            best_only = True
            write_both = False
            if verbose: print 'set best_only to ', best_only
        if o in ('-L', '--L'): #create pdbqt file for largestcluster result only
            largestCl_only = True
            write_both = False
            if verbose: print 'set largestCL_only to ', largestCL_only
        if o in ('-l', '--l'): #stem for pdbqt file for be in largest cluster
            lc_stem = a
            if verbose: print 'set largest_cluster stem to ', lc_stem

        if o in ('-x', '--x'):
            max_cl_to_write = int(a)
            if verbose: print 'set maximum number of clusters to write to ', max_cl_to_write

        if o in ('-Z', '--Z'):
            include_interactions = False
            if verbose: print 'set include_interactions to ', include_interactions

        if o in ('-n', '--n'):
            build_hbonds = False
            if verbose: print 'set build_hbonds to ', build_hbonds
        if o in ('-c', '--c'):
            detect_close_contacts = False
            if verbose: print 'set detect_close_contacts to ', detect_close_contacts
        if o in ('-p', '--p'):
            detect_pi = True
            if verbose: print 'set detect_pi interactions to ', detect_pi

        if o in ('-H', '--H'):
            custom_hb_parameters = a
            if verbose: print 'custom_hb_parameters to ', custom_hb_parameters
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to ', True
        if o in ('-h', '--'):
            usage()
            sys.exit()

    if not  directory:
        print 'process_VSResults: directory must be specified.'
        usage()
        sys.exit()

    intF = None
    if len(custom_hb_parameters):
        if verbose: print "using custom hb parameters!"
        #custom_hb_parameters "distCutoff=3.00,distCutoff2=3.25"
        hb_list = custom_hb_parameters.split(',')
        kw = {}
        for item in hb_list:
            param, value = item.split('=')
            kw[param] = float(value)
            if verbose: print param, '=', value
        hbB = apply(HydrogenBondBuilder, (), kw)
        intF = InteractionDetector(receptor_filename, detect_pi=detect_pi, hydrogen_bond_builder=hbB)

    drp = DockingResultProcessor(rms_tolerance=rms_tolerance,
                                 rms_reference=rms_reference,
                                 receptor_filename=receptor_filename,
                                 write_both=write_both,
                                 best_only=best_only,
                                 largestCl_only=largestCl_only,
                                 lc_stem=lc_stem, 
                                 max_cl_to_write=max_cl_to_write, 
                                 include_interactions=include_interactions, 
                                 detect_pi=detect_pi, 
                                 build_hbonds=build_hbonds, 
                                 detect_close_contacts=detect_close_contacts, 
                                 intF=intF) 
    drp.process(directory, verbose=verbose)




#!/usr/bin/env python
#
# 
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/compute_interatomic_distance_per_vina_pose.py,v 1.3 2013/11/14 23:14:23 rhuey Exp $
#
import os, glob
import numpy.oldnumeric as Numeric
from math import sqrt

from MolKit import Read
from MolKit.molecule import MoleculeSet
from MolKit.stringSelector import CompoundStringSelector

from AutoDockTools.Docking import Docking



def dist(coords1, coords2):
    """return distance between two atoms, a and b.
    """
    npts = len(coords1)
    pt1 = Numeric.add.reduce(coords1)/npts
    pt2 = Numeric.add.reduce(coords2)/npts
    d = Numeric.array(pt1) - Numeric.array(pt2)
    return sqrt(Numeric.sum(d*d))


if __name__ == '__main__':
    import sys
    import getopt


    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: compute_interatomic_distance_per_vina_pose.py -r hsg1:A:ASP29:N -l ind:I:IND201:N5"
        print
        print "  Description of command..."
        print "  Compute the distance between a specified receptor atom and a specified ligand atom "
        print "      for each docked pose in specified vina result file:    "
        print "      -r     full name of receptor atom, eg 'hsg1:A:ASP29:N'"
        print "      -l     full name of ligand atom in vina_result, eg 'ind_vina:I:IND201:C27'"
        print "  Optional parameters:"
        print "     [-o]    output filename default 'rec_dockedLIG_+vinafilename_dists.txt'"
        print "     [-v]    verbose output"


    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'r:l:o:vh')
    except getopt.GetoptError, msg:
        print 'compute_interatomic_distance_per_vina_pose.py: %s' %msg
        usage()
        sys.exit(2)

    # initialize required parameters
    #-r: receptor atom fullname: eg 'hsg1:A:ASP29:N'
    ratom_name =  "" 
    #-l: ligand atom fullname: eg 'ind:I:IND201:N5' @@includes vina_result filename
    latom_name =  ""
    # optional parameters
    #-o outputfilename
    outputfilename =  None  #"rec_dockedLIG_"+vina_result +"_dists.txt"
    #-v detailed output 
    verbose = None
    

    #'r:l:o:vh'
    for o, a in opt_list:
        #print "o=", o, " a=", a
        if o in ('-r', '--r'):
            ratom_name = a
            if verbose: print 'set receptor atom full_name to ', a
        if o in ('-l', '--l'):
            latom_name = a
            if verbose: print 'set ligand atom full_name to ', a
            try:
                vina_result = latom_name.split(':')[0] + ".pdbqt"
                if verbose: print 'set vina_result filename to ', a
            except:
                print "invalid ligand name ", a , "unable to find vina result_filename ", a + ".pdbqt" 
        if o in ('-o', '--o'):
            outputfilename = a
            if verbose: print 'set outputfilename to ', a
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to ', True
        if o in ('-h', '--'):
            usage()
            sys.exit()

    if not ratom_name:
        print 'compute_interatomic_distance_per_vina_pose: receptor atom full_name must be specified! eg: "hsg1:A:ASP29:N"'
        usage()
        sys.exit()

    if not latom_name:
        print 'compute_interatomic_distance_per_vina_pose: ligand atom full_name must be specified! eg: "ind:I:IND201:N5"'
        usage()
        sys.exit()

    if outputfilename is None:
        outputfilename = "rec_dockedLIG_"+vina_result +"_dists.txt"

    #process vina_result in reference directory
    msg = 'invalid vina_result filename: ' + vina_result + " not found"
    assert os.path.exists(vina_result), msg 
    ligs = Read(vina_result, "conformations")
    msg = vina_result + " contains no valid molecules"
    assert len(ligs), msg
    lig = ligs[0]
    #determine the receptor molecule to read in
    rec_file = ratom_name.split(":")[0] + ".pdbqt"
    msg = rec_file + " file not found"
    assert os.path.exists(rec_file), msg 
    recs = Read(rec_file)
    msg = rec_file + " contains no valid molecules"
    assert len(recs), msg
    rec = recs[0]
    # locate the receptor atom
    css = CompoundStringSelector()
    rec_ats = css.select(recs, ratom_name)[0]
    msg = ratom_name + " did not match exactly 1 atom in " + rec_file
    assert len(rec_ats)==1, msg
    rec_at = rec_ats[0]
    rec_at_coords = rec_at.coords
    if verbose: 
        print "found rec_at = ", rec_at.full_name(), 
        print "with initial coords = ", rec_at.coords
    # locate the ligand atom
    lig_ats = css.select(ligs, latom_name)[0]
    msg = latom_name + " did not match exactly 1 atom in " + ligs[0].name
    assert len(lig_ats)==1, msg
    lig_at = lig_ats[0]
    if verbose: 
        print "found lig_at =>", lig_at.full_name()
        print "with initial coords=", lig_at.coords
    init_coords = lig_at.coords
    #open the outputfile
    printed_rec_coords = False
    if os.path.exists(outputfilename):
        fptr = open(outputfilename, 'a')
    else:
        fptr = open(outputfilename, 'w')
        ostr= "        rec_at coords             lig_at coords         distance\n"
        fptr.write(ostr)
        ostr= "run   %12s         %12s   (Angstroms)\n"%(ratom_name, latom_name)
        fptr.write(ostr)
    if verbose:
        print " opened output file:", outputfilename
    # set the pose to Run 1
    ctr = 0 
    if verbose:
        print "about to step through ", len(lig_at._coords), " docked conformations"
    for i in range(len(lig_at._coords)):
        lig.allAtoms.setConformation(i)
        if verbose: print "now lig_at.coords=", lig_at.coords
        new_dist = dist(rec_at.coords, lig_at.coords)
        if verbose: print " new_dist =", new_dist
        if not printed_rec_coords:
            ostr = "%2d % 6.3f % 6.3f % 6.3f --% 8.3f % 8.3f % 8.3f = %8.4f \n" %(i+1, rec_at.coords[0], rec_at.coords[1], rec_at.coords[2], lig_at.coords[0], lig_at.coords[1], lig_at.coords[2], new_dist)
            printed_rec_coords = True
        else:
            ostr = "%2d                       --% 8.3f % 8.3f % 8.3f = %8.4f \n" %(i+1, lig_at.coords[0], lig_at.coords[1], lig_at.coords[2], new_dist)
        fptr.write(ostr)
        ctr += 1
    if verbose: print "wrote ", ctr , " distances to ", outputfilename
    fptr.close()
    

# To execute this command type:
# compute_interatomic_distance_per_vina_pose.py -r receptor_atom_full_name -l ligand_atom_full_name 
#           [-o  output_filename  -v verbose] 
# NOTE: -r  and -l require arguments, as does the optional argument -o while -v sets a boolean




from NetworkEditor.items import NetworkNode

from AutoDockTools.VisionInterface.Adt.autogrid_results import autogrid_results

class AutogridResURL(NetworkNode):
    """
    Creates an autogrid_results object form a URL to the autogrid results
   
    Input: URL to autogrid results
    Output: autogrid_results object that contains info about the URL to autogrid results
    """
    
    def __init__(self, name='AutogridResURL', **kw):
        kw['name'] = name
        apply( NetworkNode.__init__, (self,), kw )


        kw['name'] = name
        apply( NetworkNode.__init__, (self,), kw )
        ip = self.inputPortsDescr
        ip.append(datatype='string', name='url')

        self.widgetDescr['url'] = {
            'class':'NEEntry', 'master':'node', 'width':40,
            'labelCfg':{'text':'URL to Autogrid Results: '}
            }

        op = self.outputPortsDescr
        op.append(datatype='autogrid_results', name='autogrid_res_obj')

        code = """def doit(self, url):
    autogrid_res_obj=autogrid_results(url, "url")

    self.outputData(autogrid_res_obj=autogrid_res_obj)
"""
        self.setFunction(code)



#!/usr/bin/env python
#
# 
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/compute_interatomic_distance_per_pose.py,v 1.4 2013/11/08 17:29:16 rhuey Exp $
#
import os, glob
import numpy.oldnumeric as Numeric
from math import sqrt

from MolKit import Read
from MolKit.molecule import MoleculeSet
from MolKit.stringSelector import CompoundStringSelector

from AutoDockTools.Docking import Docking



def dist(coords1, coords2):
    """return distance between two atoms, a and b.
    """
    npts = len(coords1)
    pt1 = Numeric.add.reduce(coords1)/npts
    pt2 = Numeric.add.reduce(coords2)/npts
    d = Numeric.array(pt1) - Numeric.array(pt2)
    return sqrt(Numeric.sum(d*d))


if __name__ == '__main__':
    import sys
    import getopt


    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: compute_interatomic_distance_per_pose.py -r hsg1:A:ASP29:N -l ind:I:IND201:N5 -d ind.dlg"
        print
        print "  Description of command..."
        print "  Compute the distance between a specified receptor atom and a specified ligand atom "
        print "      for each docked pose in specified dlg file:    "
        print "      -r     full name of receptor atom "
        print "      -l     full name of ligand atom "
        print "      -d     dlg filename"
        print "  Optional parameters:"
        print "     [-o]    output filename"
        print "     [-v]    verbose output"


    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'r:l:d:o:vh')
    except getopt.GetoptError, msg:
        print 'compute_interatomic_distance_per_pose.py: %s' %msg
        usage()
        sys.exit(2)

    # initialize required parameters
    #-r: receptor atom fullname: eg 'hsg1:A:ASP29:N'
    ratom_name =  ""
    #-l: receptor atom fullname: eg 'ind:I:IND201:N5'
    latom_name =  ""
    #-d: dlg_filename
    dlg_filename =  ""
    #-o outputfilename
    outputfilename =  None #"rec_dockedLIG_"+dlg_filename +"_dists.txt"
    # optional parameters
    #-v detailed output 
    verbose = None

    #'r:l:d:o:vh'
    for o, a in opt_list:
        #print "o=", o, " a=", a
        if o in ('-r', '--r'):
            ratom_name = a
            if verbose: print 'set receptor atom full_name to ', a
        if o in ('-l', '--l'):
            latom_name = a
            if verbose: print 'set ligand atom full_name to ', a
        if o in ('-d', '--d'):
            dlg_filename = a
            if verbose: print 'set dlg filename to ', a
        if o in ('-o', '--o'):
            outputfilename = a
            if verbose: print 'set outputfilename to ', a
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to ', True
        if o in ('-h', '--'):
            usage()
            sys.exit()

    if not ratom_name:
        print 'compute_interatomic_distance_per_pose: receptor atom full_name must be specified: eg: "hsg1:A:ASP29:N"'
        usage()
        sys.exit()

    if not latom_name:
        print 'compute_interatomic_distance_per_pose: ligand atom full_name must be specified: eg: "ind:I:IND201:N5"'
        usage()
        sys.exit()

    if not dlg_filename:
        print 'compute_interatomic_distance_per_pose: autodock dlg filename must be specified: eg: "ind.dlg"'
        usage()
        sys.exit()

    if outputfilename is None:
        outputfilename = "rec_dockedLIG_"+dlg_filename +"_dists.txt"

    #process docking in reference directory
    d = Docking()
    d.readDlg(dlg_filename)
    lig = d.ligMol
    #determine the receptor molecule to read in
    rec_file = ratom_name.split(":")[0] + ".pdbqt"
    msg = rec_file + " not found"
    assert os.path.exists(rec_file), msg 
    recs = Read(rec_file)
    rec = recs[0]
    # locate the receptor atom
    css = CompoundStringSelector()
    rec_ats = css.select(recs, ratom_name)[0]
    msg = ratom_name + " did not match exactly 1 atom in " + rec_file
    assert len(rec_ats)==1, msg
    rec_at = rec_ats[0]
    rec_at_coords = rec_at.coords
    if verbose: 
        print "found rec_at = ", rec_at.full_name(), 
        print "with initial coords = ", rec_at.coords
    # locate the ligand atom
    lig_ats = css.select(MoleculeSet([d.ligMol]), latom_name)[0]
    assert len(lig_ats)==1
    lig_at = lig_ats[0]
    print "found lig_at =>", lig_at.full_name()
    print "initial coords=", lig_at.coords
    init_coords = lig_at.coords
    #open the outputfile
    if os.path.exists(outputfilename):
        fptr = open(outputfilename, 'a')
    else:
        fptr = open(outputfilename, 'w')
        ostr= "run     rec_at coords             lig_at coords        distance \n"
        fptr.write(ostr)
    print " opened output file:", outputfilename
    # set the pose to Run 1
    ctr = 1 
    print "about to step through ", len(d.ch.conformations), " docked conformations"
    for i in range(len(d.ch.conformations)):
        d.ch.set_conformation(d.ch.conformations[i])
        if verbose: print "now lig_at.coords=", lig_at.coords
        new_dist = dist(rec_at.coords, lig_at.coords)
        if verbose: print " new_dist =", new_dist
        ostr = "%2d % 6.3f % 6.3f % 6.3f --% 8.3f % 8.3f % 8.3f = %8.4f \n" %(i+1, rec_at.coords[0], rec_at.coords[1], rec_at.coords[2], lig_at.coords[0], lig_at.coords[1], lig_at.coords[2], new_dist)
        fptr.write(ostr)
        ctr += 1
    if verbose: print "wrote ", ctr , " lines to ", outputfilename
    fptr.close()
    

# To execute this command type:
# compute_interatomic_distance_per_pose.py -r receptor_atom_full_name -l ligand_atom_full_name -d dlg filename 
#           -o  output_filename  -v verbose
# NOTE: -r, -l and -d require arguments, as does the optional argument -o while -v sets a boolean




#########################################################################
#
# Date: Nov 2001 Authors: Michel Sanner
#
#    sanner@scripps.edu
#
#       The Scripps Research Institute (TSRI)
#       Molecular Graphics Lab
#       La Jolla, CA 92037, USA
#
# Copyright: Michel Sanner and TSRI
#
#########################################################################

from NetworkEditor.items import NetworkNode

from AutoDockTools.VisionInterface.Adt.LigandDB import LigandDB

import sys
import os

class UrlLigandDB(NetworkNode):
    """
    Input: URL to a ligand DB on the virtual screening server.
    Output: LigandDB object containing info about the input
    """
    
    def __init__(self, name='UrlLigandDB', **kw):
        kw['name'] = name
        apply( NetworkNode.__init__, (self,), kw )


        kw['name'] = name
        apply( NetworkNode.__init__, (self,), kw )
        ip = self.inputPortsDescr
        ip.append(datatype='string', name='url')
#        ip.append(datatype='string', name='local_dir')

        self.widgetDescr['url'] = {
            'class':'NEEntry', 'master':'node', 'width':40,
            'labelCfg':{'text':'URL to ligands: '}
            }
#        self.widgetDescr['local_dir'] = {
#            'class':'NEEntryWithDirectoryBrowser', 'master':'node', 'width':30,
#            'labelCfg':{'text':'Local directory to save ligands: '}
#            }

        op = self.outputPortsDescr
        op.append(datatype='LigandDB', name='ligDB')

#        code = """def doit(self, url, local_dir):
        code = """def doit(self, url):
#    ligDB = LigandDB(url_compressed_file=url)
    ligDB = LigandDB(url_lib=url)

    bname = os.path.basename(url)

#    if bname.endwith('.tar.gz'):
#        print "TEST tar.gz"
#    elif bname.endswith('.zip'):
#        print "TEST ZIP"

    if not(url.startswith('http://kryptonite.nbcr.net/app')):
         print "ERROR: the URL must be a directory on kryptonite.nbcr.net"
         sys.exit()  
 
    

    self.outputData(ligDB=ligDB)
"""
        self.setFunction(code)



## Automatically adapted for numpy.oldnumeric Jul 23, 2007 by 

#!/usr/bin/env python
#
# $Id: pixelMap2D.py,v 1.4 2007/10/08 18:16:25 rhuey Exp $

from DejaVu.colorTool import TkColor, RedWhiteRamp
from DejaVu.colorMap import ColorMap
import Image, numpy.oldnumeric as Numeric


class PixelMap2D:
    """
    PixelMap2D:
        Used to display 2D representations of 2D Numeric arrays of data, colored by magnitude.
        Its cmap, a DejaVu.colorMap, provides a mapping from each data
        value to an entry in a 'ramp' which is a list of lists of 3 floats between 0-1. 
        (By default, a RedWhiteRamp is used. Each entry defines a color for that value.)  
        The property range is set by default from the 'mini' and 'maxi' values of the
        results array but it can be set by specifying values for 'mini' and 'maxi'. In
        the latter case, the values of the result array are clamped to be
        greater than 'mini' and less than 'maxi'. A copy of the original data is used.

        Its method to display data "show_image" -> [x,y] produces an array of pixels 
        colored by magnitude of corresponding x,y result and is implemented using 
        PIL.Image show_image.
        "On Unix platforms, this saves the image to a temporary PPM file, and
        calls the xv utility.  On Windows, it saves the image to a temporary BMP file, 
        and uses the standard BMP display utility to show it.""  
        (F.Lundh, http://effbot.org/imagingbook/image.htm)

        Its method to display the corresponding color map legend "show_legend"
        produces an array of pixels, 1-by-nsteps, showing the range of the
        colormap.  This legend is displayed using xv on Unix platforms
    """

    def __init__(self, results, master=None, mini=None, maxi=None, 
                    cmap=None, cmap_name='cmap', ramp=None, npixels=10):
        assert isinstance(results, type(Numeric.array(range(4))))
        assert len(results.shape)==2
        # a copy of the input array is used
        self.results = Numeric.array(results[:])
        self.results = results
        self.master = master
        #for a Numeric array,
        self.x_number = results.shape[0]   #number of rows 
        self.y_number = results.shape[1]   #number of columns
        self.results = results
        self.npixels = npixels
        if mini is None:
            mini=min(results.ravel())
        else:
            for x in xrange(self.x_number):
                for y in xrange(self.y_number):
                    if self.results[x][y]<mini:
                        self.results[x][y]=mini
        self.mini = mini
        if maxi is None:
            maxi=max(results.ravel())
        else:
            for x in xrange(self.x_number):
                for y in xrange(self.y_number):
                    if self.results[x][y]>maxi:
                        self.results[x][y]=maxi
        self.maxi = maxi
        self.cmap = cmap 
        if cmap is None:
            if ramp is None:
                ramp = RedWhiteRamp()
            self.cmap = ColorMap(cmap_name, mini=mini, maxi=maxi, ramp=ramp)

    def show_image(self):
        #'RGB' requires a tuple (r,g,b) where r,g,b are ints in range 0-255
        #'RGBA' requires a tuple (r,g,b,a) where r,g,b,a are ints in range 0-255
        values = self.cmap.Map(self.results.ravel())
        self.image = Image.fromstring('RGBA', (self.x_number,self.y_number), 
                                (values*255).astype('B').tostring())
        self.image.show()


    def show_legend(self, steps=20, horizontal=True):
        delta = (self.maxi-self.mini)/(steps-1)
        vals = self.vals= []
        for i in range(steps-1):
            vals.append(self.mini+i*delta)
        self.vals.append(self.maxi)
        values = self.cmap.Map(vals)
        if horizontal:
            self.legend = Image.fromstring('RGBA', (steps, 1), (values*255).astype('B').tostring())
        else:
            self.legend = Image.fromstring('RGBA', (1,steps), (values*255).astype('B').tostring())
        self.legend.show()


      
if __name__=='__main__':
    import numpy.oldnumeric as Numeric
    from time import sleep
    r = Numeric.array(range(10000))
    r.shape = (100,100)

    rw_w = PixelMap2D(r[:])
    rw_w.show_image()
    print "rw_w shown"
    sleep(3)


    rw_half_w = PixelMap2D(Numeric.array(r[:])[:], maxi=5000.)
    rw_half_w.show_image()
    print "rw_half_w shown"
    sleep(3)

    from DejaVu.colorTool import RedWhiteBlueRamp
    rwb_w = PixelMap2D(r[:], ramp=RedWhiteBlueRamp())
    rwb_w.show_image()
    print "rwb_w shown"
    sleep(3)

    s_r = Numeric.array(Numeric.swapaxes(r[:],0,1))
    s_r.shape = (100,100)
    s_r_w = PixelMap2D(s_r, ramp=RedWhiteBlueRamp())
    s_r_w.show_image()
    print "s_r_w shown"
    sleep(3)

    
    rwb_half_w = PixelMap2D(Numeric.array(r[:])[:], maxi=5000., ramp=RedWhiteBlueRamp())
    rwb_half_w.show_image()
    print "rwb_half_w shown"
    sleep(3)


## this example depends on finding data in working_directory_file 'ligand_results.py'
## stored as a dictionary 'ligand_results'
##stems = ['1a30', '1a94', '1a9m', '1aaq', '1ajv', '1ajx', '1b11', '1b6j', '1b6k',
##    '1b6l', '1b6m', '1b6n', '1b6o', '1b6p', '1bdl', '1bdq', '1bdr', '1bv7',
##    '1bv9', '1bwa', '1bwb', '1d4k', '1d4l', '1dmp', '1fiv', '1fmb', '1g2k',
##    '1g35', '1gnm', '1gnn', '1gno', '1hbv', '1hef', '1heg', '1hih', '1hii',
##    '1hiv', '1hos', '1hpo', '1hps', '1hpv', '1hpx', '1hsg', '1hsh', '1hte',
##    '1htf', '1htg', '1hvh', '1hvi', '1hvj', '1hvk', '1hvl', '1hvr', '1hvs',
##    '1hwr', '1hxw', '1iiq', '1izh', '1izi', '1jld', '1k6c', '1k6p', '1k6t',
##    '1k6v', '1kzk', '1mes', '1met', '1meu', '1mtr', '1mui', '1ody', '1pro',
##    '1qbr', '1qbs', '1qbt', '1qbu', '1sbg', '1tcw', '1tcx', '1vij', '1vik',
##    '2bpv', '2bpx', '2fmb', '2mip', '2upj', '3aid', '3tlh', '4fiv', '4hvp',
##    '4phv', '5fiv', '5hvp', '5upj', '6fiv', '6upj', '7hvp', '7upj', '8hvp', '9hvp']
##from ligand_results import ligand_results
##from receptor_results import receptor_results
##all_ligand_results = []
##all_receptor_results = []
##for stem in stems: 
##    all_ligand_results.extend(ligand_results[stem])
##    all_receptor_results.extend(receptor_results[stem])
##here results are [lig1,lig2,lig3,lig4...]
## here rows in image will correspond to ligands and columns to receptors

##import numpy.oldnumeric as Numeric
##ligand_results = Numeric.array(all_ligand_results)
##ligand_results.shape = (100,100)
##now ligand_results are #[[lig1]
                         #[lig2]
                         #[lig3]
                         #[lig4]
                         #...]
##receptor_results = Numeric.array(all_receptor_results)
##receptor_results.shape = (100,100)
##now receptor_results are #[[rec1]
                           #[rec2]
                           #[rec3]
                           #[rec4]
                           #...]
## here rows in image will correspond to receptors and columns to ligands
##mini = min(results.ravel())
###clamp upper bound to 0
##maxi = 0
## from pixelMap2D import PixelMap2D
##rw = PixelMap2D(results, mini=mini, maxi=maxi)
##rw.show_image(zero_color=(255, 255, 255))


        



########################################################################
#
#    Vision Node - Python source code - file generated by vision
#    Monday 22 June 2009 13:08:37 
#    
#       The Scripps Research Institute (TSRI)
#       Molecular Graphics Lab
#       La Jolla, CA 92037, USA
#
# Copyright: Daniel Stoffler, Michel Sanner and TSRI
#   
# revision: Guillaume Vareille
#  
#########################################################################
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/VisionInterface/Adt/Mapper/ADTFileNames.py,v 1.3 2010/07/15 23:45:39 jren Exp $
#
# $Id: ADTFileNames.py,v 1.3 2010/07/15 23:45:39 jren Exp $
#

# import node's base class node
from NetworkEditor.items import NetworkNode
class ADTFileNames(NetworkNode):
    """
    Node to make inputs for virtual screening.
    """
    mRequiredTypes = {}
    mRequiredSynonyms = [
    ]

    def __init__(self, constrkw = {},  name='ADTFileNames', **kw):
        kw['constrkw'] = constrkw
        kw['name'] = name
        apply( NetworkNode.__init__, (self,), kw)

        ip = self.inputPortsDescr
        ip.append({'name': 'recpetor_obj', 'datatype': 'receptor'})

        op = self.outputPortsDescr
        op.append({'name': 'GPF_template', 'datatype': 'gpf_template'})
        op.append({'name': 'DPF_template', 'datatype': 'dpf_template'})
        op.append({'name': 'result_dir', 'datatype': 'string'})


        code = """def doit(self, receptor_obj):
        import os
        from AutoDockTools.VisionInterface.Adt.receptor import receptor
        from AutoDockTools.VisionInterface.Adt.dpf_template import dpf_template
        from AutoDockTools.VisionInterface.Adt.gpf_template import gpf_template

        receptor_id = receptor_obj.get_id()
        receptor_dir = receptor_obj.get_workdir() 

        GPF_template = gpf_template(receptor_dir + os.sep + receptor_id + '.gpf')
        DPF_template = dpf_template(receptor_dir + os.sep + receptor_id + '.dpf')
        result_dir = os.path.abspath(receptor_dir + os.sep + '..' + os.sep + receptor_id)

        print "-------------------------------------------------------"
        print "     INPUTS THAT WILL BE USED FOR VIRTUAL SCREENING    "
        print "GPF Template:                  " + GPF_template.fullpath
        print "DPF Template:                  " + DPF_template.fullpath
        print "Results will be downloaded to: " + result_dir
        print "-------------------------------------------------------"
        
        pass
        self.outputData(GPF_template=GPF_template, DPF_template=DPF_template, result_dir=result_dir)
"""
        self.configure(function=code)


    def beforeAddingToNetwork(self, net):
        try:
            ed = net.getEditor()
        except:
            import traceback; traceback.print_exc()
            print 'Warning! Could not import widgets'




#
#
#  $Id: test_AD41dpf.py,v 1.8 2014/04/21 17:34:26 rhuey Exp $
#
#


import unittest, glob, os
import time
from string import split, find, strip

mv = None
ct = 0
totalCt = 42


class AD41dpf_BaseTests(unittest.TestCase):
    """
    setUp + tearDown form a fixture: working environment for the testing code
    """
    
        
    def startViewer(self):
        global mv
        if mv is None:
            from MolKit import Read
            import Tkinter
            from Pmv.moleculeViewer import MoleculeViewer
            mv = MoleculeViewer(trapExceptions=False)
            mv.loadModule('deleteCommands', 'Pmv')
            mv.loadModule('selectionCommands', 'Pmv')
            #change warningMsg format
            mv.setUserPreference(('warningMsgFormat', 'printed'), log = 0)
            mv.GUI.toolbarCheckbuttons['ADT']['Variable'].set(1)
            mv.Add_ADT()
            mv.loadModule('autodpfCommands', 'AutoDockTools')
            from AutoDockTools import setADTmode
            setADTmode("AD4.2", mv)
        self.mv = mv


    def setUp(self):
        """
        clean-up
        """
        if not hasattr(self, 'mv'):
            self.startViewer()


    def tearDown(self):
        """
        clean-up
        """
        #print 'in gpf tearDown'
        global ct, totalCt
        #reset docked
        from AutoDockTools.DockingParameters import DockingParameters
        self.mv.dpo = DockingParameters()
        self.mv.dpo.vf = self.mv
        from AutoDockTools.GridParameters import GridParameters
        self.mv.gpo = GridParameters()
        self.mv.gpo.vf=self.mv
        self.mv.DPF_LIGAND_TYPES = []
        self.mv.DPF_FLEXRES_TYPES = []
        #delete any molecules left due to errors
        for m in self.mv.Mols:
            self.mv.deleteMol(m)
        ct = ct + 1
        if ct==totalCt:
            print 'destroying mv'
            self.mv.Exit(0)
            del self.mv            
    

    def SA_GA_LS_GALS(self):    
        self.mv.AD41dpf_writeSA("ref41_SA.dpf")
        self.mv.AD41dpf_writeGA("ref41_GA.dpf")
        self.mv.AD41dpf_writeLS("ref41_LS.dpf")
        self.mv.AD41dpf_writeGAlS("ref41_GALS.dpf")



class AD41dpf_readDPFTests(AD41dpf_BaseTests):
    
    
    def test_dpf_readdpf(self):
        """reads a dpf and checks
        """ 
        filename = 'ref_1ebg.dpf'
        self.mv.ADdpf_read(filename)
        c=self.mv.dpo
        self.assertEqual(c.dpf_filename, filename)


    def test_dpf_read_empty_dpf(self):
        """checks empty dpf entry
        """
        self.assertRaises(IOError, self.mv.ADdpf_read, " ")


    def test_dpf_read_non_existent_dpf(self):
        """checks non-existent dpf
        """
        self.assertRaises(IOError, self.mv.ADdpf_read, "asd.hfd.dpf")
    
   
    def test_dpf_set_macro_molecule_file_name(self):
        """checks  macro mols filename
        """
        filename = "1ebg_rec.pdbqt"
        self.mv.AD41dpf_readMacromolecule(filename)
        self.assertEqual(self.mv.dpo.receptor_filename, filename)
             

    def test_dpf_read_empty_macro_mol(self):
   
        """checks empty macro mol entry
        """
        self.assertRaises(IOError, self.mv.AD41dpf_readMacromolecule, " ")


    def test_dpf_read_non_existent_macro_mol(self):
        """checks non-existent macro mol        """
        self.assertRaises(IOError, self.mv.AD41dpf_readMacromolecule, "asd.hfd.dpf")
    

    def test_dpf_replace_random(self):
        """ checks by setting some values in place of random
        """
        self.mv.ADdpf_read('ref41_1ebg.dpf')
        self.mv.AD41dpf_readFormattedLigand("1ebg_lig.pdbqt")
        teststring = "1.0 2.0 3.0"
        self.mv.ADdpf_setDpo(tran0=teststring)
        testfile = "whateveryouwant.dpf"
        self.mv.AD41dpf_writeGALS(testfile)
        fptr=open(testfile)
        alllines=fptr.readlines()
        for l in alllines:
            if find(l, 'tran0')==0:
                break
        ll = split(l)
        testsplit = split(teststring)
        for i in range(1, 4):
            self.assertEqual(ll[i], testsplit[i-1])
   

    def test_chosen_ligand_name(self):
        
        """checks chosen ligand name
        """
        self.mv.readMolecule('1ebg_lig.pdbqt')
        lig_name = self.mv.Mols[0].name
        self.mv.AD41dpf_chooseFormattedLigand(lig_name)
        self.assertEqual(self.mv.dpo.ligand.name, lig_name)
        

    def test_dpf_set_ligand_parameters(self):
        """ checks ligand name, ndihe, center, types
        """
        filename = '1ebg_lig.pdbqt'
        self.mv.readMolecule(filename)
        m=self.mv.Mols[0]
        self.mv.AD41dpf_chooseFormattedLigand(m.name)
        #4assertions: move is ligand.parser.filename
        #ndihe == ligand.ndihe
        #types == ligand.types
        #about==ligand.center
        mcen = []
        for v in m.center:
            mcen.append(round(v,3))
        self.assertEqual(m.ndihe, self.mv.dpo['ndihe']['value'])
        self.assertEqual(m.types, self.mv.dpo['types']['value'])
        self.assertEqual(mcen, self.mv.dpo['about']['value'])
        #self.assertEqual(m.center, self.mv.dpo['about']['value'])
        self.assertEqual(filename, self.mv.dpo.ligand_filename)
   

    def test_dpf_read_empty_ligand(self):
        """checks empty ligand
        """
        self.assertRaises(IOError, self.mv.AD41dpf_readFormattedLigand, " ")


    def test_dpf_read_non_existent_ligand(self):
        """checks non-existent ligand
        """
        self.assertRaises(IOError, self.mv.AD41dpf_readFormattedLigand, "asd.hfd.dpf")

    
#    def test_dpf_read_ligand_widget_exists(self):
#        """checks for widget when read ligand called
#        """
#        self.mv.AD41dpf_readFormattedLigand('1ebg_lig.pdbqt')
#        c = self.mv.AD41dpf_initLigand
#        self.assertEqual(c.form.root.winfo_ismapped(), 1)
#        self.assertEqual(c.form.root.withdraw(),'')

     
    def test_search_parameters_simulated_annealing_replace(self):
        """checks after replacing
        """
        self.mv.ADdpf_setDpo(rtrf='0.5')
        self.assertEqual(self.mv.dpo['rtrf']['value'],'0.5')
        

    def test_search_parameters_ga_parameters_change(self):
        """checks after replacing
        """
        d={}
        d['ga_mutation_rate'] =0.5 
        apply(mv.ADdpf_setGAparameters, (), d)
        self.assertEqual(self.mv.dpo['ga_mutation_rate']['value'],0.5)
               
    
    def test_search_parameters_ga_parameters_values(self):
        """checks values in widget and file are same
        """
        self.mv.AD41dpf_readFormattedLigand("1ebg_lig.pdbqt")
        self.mv.AD41dpf_readMacromolecule("1ebg_rec.pdbqt")
        testfile = "somename.dpf"
        self.mv.AD41dpf_writeGALS(testfile)
        fptr = open(testfile)
        alllines = fptr.readlines()
        fptr.close()
        ll = [0, 0]
        for i in alllines:        
            if find(i,'ga_window_size')==0:
                ll=split(i)
                break
        self.assertEqual(str(mv.dpo['ga_window_size']['value']), ll[1])
    

    def test_search_parameters_ga_parameters_replace(self):
        """checks in file after replacing in widget
        """
        self.mv.AD41dpf_readFormattedLigand("1ebg_lig.pdbqt")
        self.mv.AD41dpf_readMacromolecule("1ebg_rec.pdbqt")
        d={}
        d['ga_pop_size'] = 2
        apply(self.mv.ADdpf_setGAparameters, (), d)
        testfile = "somename.dpf"
        self.mv.AD41dpf_writeGALS(testfile)
        fptr=open(testfile)
        alllines=fptr.readlines()
        fptr.close()
        ll = [0, 0]
        for i in alllines:        
            if find(i,'ga_pop_size')==0:
                ll=split(i)
                break
        self.assertEqual(ll[1],'2')
        
   
    def test_search_parameters_ls_parameters_values(self):
        """checks after replacing ls parameter
        """
        d={}
        d['ls_search_freq'] =0.5 
        apply(mv.ADdpf_setLSparameters, (), d)
        self.assertEqual(self.mv.dpo['ls_search_freq']['value'],0.5)
   

    def test_search_parameters_ls_parameters_replace(self):
        """checks ls  paarmeters in widget and file are same
        """
        self.mv.AD41dpf_readFormattedLigand("1ebg_lig.pdbqt")
        self.mv.AD41dpf_readMacromolecule("1ebg_rec.pdbqt")
        d={}
        d['ls_search_freq'] = 0.02 
        apply(mv.ADdpf_setLSparameters, (), d)
        testfile = "somename.dpf"
        self.mv.AD41dpf_writeGALS(testfile)
        fptr=open(testfile)
        alllines=fptr.readlines()
        ll = [0, 0]
        for i in alllines:        
            if find(i,'ls_search_freq')==0:
                ll=split(i)
        self.assertEqual(str(round(mv.dpo['ls_search_freq']['value'],2)),ll[1])
        

    def test_search_parameters_ls_parameters_change(self):
        """checks in file after replacing ls parameters in widget
        """
        self.mv.AD41dpf_readFormattedLigand("1ebg_lig.pdbqt")
        self.mv.AD41dpf_readMacromolecule("1ebg_rec.pdbqt")
        d={}
        d['ls_search_freq'] = 0.02
        apply(mv.ADdpf_setLSparameters, (), d)
        testfile = "somename.dpf"
        self.mv.AD41dpf_writeGALS(testfile)
        fptr=open(testfile)
        alllines=fptr.readlines()
        fptr.close()
        ll = [0, 0]
        for i in alllines:        
            if find(i,'ls_search_freq')==0:
                ll=split(i)
                break
        self.assertEqual(ll[1],'0.02')
   

    def test_search_parameters_docking_parameters_values(self):
        """checks docking parameters after replacing
        """
        d={}
        d['rmstol'] = 4.0 
        apply(mv.ADdpf_setDockingParameters, (), d)
        self.assertEqual(self.mv.dpo['rmstol']['value'], 4.0)


    def test_search_parameters_docking_parameters_replace(self):
        """checks docking  paarmeters in widget and file are same
        """
        self.mv.AD41dpf_readFormattedLigand("1ebg_lig.pdbqt")
        self.mv.AD41dpf_readMacromolecule("1ebg_rec.pdbqt")
        test_val = 213        
        d={}
        d['sw_max_fail'] =  test_val 
        apply(mv.ADdpf_setDockingParameters, (), d)
        testfile = "somename.dpf"
        self.mv.AD41dpf_writeGALS(testfile)
        fptr=open(testfile)
        alllines=fptr.readlines()
        fptr.close()
        ll = [0,0]
        for line in alllines:        
            if find(line,'sw_max_fail')>-1:
                ll=split(line)
                break
        self.assertEqual(test_val,int(ll[1]))
        self.assertEqual(self.mv.dpo['sw_max_fail']['value'],test_val)
         

    def test_search_parameters_docking_parameters_change(self):
        """checks in file after replacing ls parameters in widget
        """
        self.mv.AD41dpf_readFormattedLigand("1ebg_lig.pdbqt")
        self.mv.AD41dpf_readMacromolecule("1ebg_rec.pdbqt")
        test_val = 312
        d={}
        d['sw_max_its'] =  test_val
        apply(mv.ADdpf_setDockingParameters, (), d)
        testfile = "somename.dpf"
        self.mv.AD41dpf_writeGALS(testfile)
        fptr=open(testfile)
        alllines=fptr.readlines()
        fptr.close()
        ll = [0,0]
        for i in alllines:        
            if find(i,'sw_max_its')==0:
                ll=split(i)
                break
        self.assertEqual(int(ll[1]),test_val)


    def test_sa_dpf_compare_lines(self):
        
        """checks ref41_SA file and test_SA file are same
        """
        self.mv.AD41dpf_readFormattedLigand("1ebg_lig.pdbqt")
        self.mv.AD41dpf_readMacromolecule("1ebg_rec.pdbqt")
        self.mv.AD41dpf_writeSA("test_SA.dpf")
        testptr = open("test_SA.dpf")
        testlines = testptr.readlines()
        refptr = open("ref41_SA.dpf")
        reflines = refptr.readlines()
        refptr.close()
        for refline,testline in zip(reflines,testlines):
            self.assertEqual(refline.split(),testline.split())
            

    def test_ga_dpf_compare_lines(self):
        
        """checks ref41_GA file and test_GA file are same
        """
        self.mv.AD41dpf_readFormattedLigand("1ebg_lig.pdbqt")
        self.mv.AD41dpf_readMacromolecule("1ebg_rec.pdbqt")
        self.mv.AD41dpf_writeGA("test_GA.dpf")
        testptr = open("test_GA.dpf")
        testlines = testptr.readlines()
        refptr = open("ref41_GA.dpf")
        reflines = refptr.readlines()
        refptr.close()
        for refline,testline in zip(reflines,testlines):
            self.assertEqual(refline.split(),testline.split())


    def test_ls_dpf_compare_lines(self):
        
        """checks ref41_LS file and test_LS file are same
        """
        self.mv.AD41dpf_readFormattedLigand("1ebg_lig.pdbqt")
        self.mv.AD41dpf_readMacromolecule("1ebg_rec.pdbqt")
        self.mv.AD41dpf_writeLS("test_LS.dpf")
        testptr = open("test_LS.dpf")
        testlines = testptr.readlines()
        refptr = open("ref41_LS.dpf")
        reflines = refptr.readlines()
        refptr.close()
        for refline,testline in zip(reflines,testlines):
            self.assertEqual(refline.split(),testline.split())


    def test_gals_dpf_compare_lines(self):
        
        """checks ref41_GALS file and test_GALS file are same
        """
        self.mv.AD41dpf_readFormattedLigand("1ebg_lig.pdbqt")
        self.mv.AD41dpf_readMacromolecule("1ebg_rec.pdbqt")
        self.mv.AD41dpf_writeGALS("test_GALS.dpf")
        testptr = open("test_GALS.dpf")
        testlines = testptr.readlines()
        refptr = open("ref41dpf_GALS.dpf")
        reflines = refptr.readlines()
        refptr.close()
        for refline,testline in zip(reflines,testlines):
            self.assertEqual(refline.split(),testline.split())



class AD41dpf_flexResTests(AD41dpf_BaseTests):

    #read receptor
    def test_dpf4_select_rigid_macro_mol(self):
        """select a rigid receptor and checks dpo receptor_filename 
        """
        filename = 'hsg1_rigid.pdbqt'
        self.mv.AD41dpf_readMacromolecule(filename)
        self.assertEqual(self.mv.dpo.receptor_filename,filename)


    #read ligand
    def test_dpf4_read_flex_mol(self):
        """reads flexible input file and checks dpo ligand_filename 
        """
        filename = 'hsg1_flex.pdbqt'
        self.mv.AD41dpf_readFormattedLigand(filename)
        ligand_name = self.mv.Mols[-1].name
        self.assertEqual(self.mv.dpo.ligand.name, ligand_name)
        self.assertEqual('ARG8' in self.mv.Mols[-1].chains.residues.name, True)
        #print "self.mv.Mols[-1].name=", self.mv.Mols[0].name
        args = self.mv.Mols[-1].chains.residues.get(lambda x:x.name=='ARG8')
        self.assertEqual(len(args), 2)
        self.assertEqual(len(args.atoms.bonds[0])>0, True)
    

    #choose ligand
    def test_dpf4_choose_flex_mol(self):
        """read then choose flexible input file; check dpo ligand_filename 
        """
        filename = 'hsg1_flex.pdbqt'
        self.mv.readMolecule(filename)
        ligand_name = self.mv.Mols[-1].name
        self.assertEqual('ARG8' in self.mv.Mols[-1].chains.residues.name, True)
        args = self.mv.Mols[-1].chains.residues.get(lambda x:x.name=='ARG8')
        self.assertEqual(len(args), 2)
        self.assertEqual(len(args.atoms.bonds[0])>0, True)
        self.mv.AD41dpf_chooseFormattedLigand(ligand_name)
        self.assertEqual(self.mv.dpo.ligand.name, ligand_name)



class AD41dpf_formatting_flexResTests(AD41dpf_BaseTests):

    def setUp(self):
        """
        clean-up
        """
        if not hasattr(self, 'mv'):
            self.startViewer()
        for m in self.mv.Mols:
            self.mv.deleteMol(m)
        self.mv.loadModule('autoflexCommands', 'AutoDockTools')
        self.mv.readMolecule('hsg1.pdbqt', ask=0, parser=None)
        self.mv.ADflex_chooseMacro("hsg1")
        from MolKit.molecule import Atom
        self.mv.setIcomLevel(Atom, KlassSet=None)
        self.mv.ADflex_setResidues("::ARG8")
        self.mv.ADflex_setBondRotatableFlag("hsg1:B:ARG8:NE,CZ;", False)
        self.mv.ADflex_setBondRotatableFlag("hsg1:B:ARG8:CZ,NH1;", False)
        self.mv.ADflex_setBondRotatableFlag("hsg1:B:ARG8:CZ,NH2;", False)
        self.mv.ADflex_setBondRotatableFlag("hsg1:A:ARG8:NE,CZ;", False)
        self.mv.ADflex_setBondRotatableFlag("hsg1:A:ARG8:CZ,NH1;", False)
        self.mv.ADflex_setBondRotatableFlag("hsg1:A:ARG8:CZ,NH2;", False)
        self.mv.ADflex_setBondRotatableFlag("hsg1:B:ARG8:CA,CB;", False)
        self.mv.ADflex_setBondRotatableFlag("hsg1:A:ARG8:CA,CB;", False)
        self.mv.ADflex_writeFlexFile('temp_FLEX.pdbqt')
        self.mv.ADflex_writeRigidFile('temp_RIGID.pdbqt')


    #choose receptor
    def test_dpf4_choose_rigid_mol(self):
        """choose new rigid file. check dpo macro_filename 
        """
        written_macro_name = "temp_RIGID.pdbqt"
        self.mv.AD41dpf_readMacromolecule(written_macro_name)
        macro_name = "temp_RIGID"
        self.assertEqual(self.mv.dpo.receptor_stem, macro_name)
        self.assertEqual(self.mv.dpo['fld']['value'], macro_name+".maps.fld")



class AD41dpf_FlexRes_Lig_Order_Tests(AD41dpf_BaseTests):

    def test_gpf4_lig_flexres(self):
        """flexResTests: read a ligand file then a flexres file and check written types
        """
        rigid_filename = 'CathepsinB_rigid.pdbqt'
        flexres_filename = 'CathepsinB_flex.pdbqt'
        ligand_filename = 'FGA3.pdbqt'
        output_filename = 'lig_flexres.dpf'
        self.mv.AD41dpf_readMacromolecule(rigid_filename)
        self.mv.AD41dpf_readFormattedLigand(ligand_filename)
        self.mv.AD41dpf_readFlexRes(flexres_filename)
        self.mv.AD41dpf_writeGALS(output_filename, log=0)
        fptr = open(output_filename)
        lines = fptr.readlines()
        types_line = lines[4]
        ref_line = "ligand_types A C OA HD N SA          # atoms types in ligand\n"
        self.assertEqual(types_line,ref_line)


    def test_gpf4_flexres_lig(self):
        """flexResTests: read a flexres file then a ligand file and check written types
        """
        rigid_filename = 'CathepsinB_rigid.pdbqt'
        flexres_filename = 'CathepsinB_flex.pdbqt'
        ligand_filename = 'FGA3.pdbqt'
        output_filename = 'flexres_lig.dpf'
        self.mv.AD41dpf_readMacromolecule(rigid_filename)
        self.mv.AD41dpf_readFlexRes(flexres_filename)
        self.mv.AD41dpf_readFormattedLigand(ligand_filename)
        self.mv.AD41dpf_writeGALS(output_filename)
        fptr = open(output_filename)
        lines = fptr.readlines()
        types_line = lines[4]
        ref_line = "ligand_types A C OA HD N SA          # atoms types in ligand\n"
        self.assertEqual(types_line,ref_line)


    
    
    def test_rig_flex_lig(self):
        """set rigid filename, flexfilename and open ligand
        """ 
        self.mv.AD41dpf_readMacromolecule('./hsg1_rigid.pdbqt', log=0)
        self.mv.AD41dpf_readFlexRes('./hsg1_FE.pdbqt', log=0)
        self.assertEqual(self.mv.DPF_FLEXRES_TYPES, ['FE'])
        self.mv.AD41dpf_readFormattedLigand('./ind_zero.pdbqt', log=0)
        self.assertEqual(self.mv.DPF_LIGAND_TYPES, ['C'])
        self.assertEqual(self.mv.DPF_FLEXRES_TYPES, ['FE'])
        self.mv.AD41dpf_writeGALS('./zero_FE_rig_GALS.dpf', log=0)
        c=self.mv.dpo
        self.assertEqual(c['ligand_types']['value'], 'C FE')

    
    def test_rig_lig_flex(self):
        """set rigid filename, open ligand and set flexfilename
        """ 
        self.mv.AD41dpf_readMacromolecule('./hsg1_rigid.pdbqt', log=0)
        self.mv.AD41dpf_readFormattedLigand('./ind_zero.pdbqt', log=0)
        self.assertEqual(self.mv.DPF_LIGAND_TYPES, ['C'])
        self.mv.AD41dpf_readFlexRes('./hsg1_FE.pdbqt', log=0)
        self.assertEqual(self.mv.DPF_FLEXRES_TYPES, ['FE'])
        self.mv.AD41dpf_writeGALS('./zero_FE_rig_GALS.dpf', log=0)
        c=self.mv.dpo
        self.assertEqual(c['ligand_types']['value'], 'C FE')

    
    def test_flex_rig_lig(self):
        """set  flex filename, rigid filename and open ligand
        """ 
        self.mv.AD41dpf_readFlexRes('./hsg1_FE.pdbqt', log=0)
        self.assertEqual(self.mv.DPF_FLEXRES_TYPES, ['FE'])
        self.mv.AD41dpf_readMacromolecule('./hsg1_rigid.pdbqt', log=0)
        self.mv.AD41dpf_readFormattedLigand('./ind_zero.pdbqt', log=0)
        self.assertEqual(self.mv.DPF_LIGAND_TYPES, ['C'])
        c=self.mv.dpo
        self.mv.AD41dpf_writeGALS('./zero_rig_GALS.dpf', log=0)
        self.assertEqual(c['ligand_types']['value'], 'C FE')

    
    def test_flex_lig_rig(self):
        """set  flex filename, open ligand and set rigid filename
        """ 
        self.mv.AD41dpf_readFlexRes('./hsg1_FE.pdbqt', log=0)
        self.assertEqual(self.mv.DPF_FLEXRES_TYPES, ['FE'])
        self.mv.AD41dpf_readFormattedLigand('./ind_zero.pdbqt', log=0)
        self.assertEqual(self.mv.DPF_LIGAND_TYPES, ['C'])
        self.mv.AD41dpf_readMacromolecule('./hsg1_rigid.pdbqt', log=0)
        c=self.mv.dpo
        self.mv.AD41dpf_writeGALS('./zero_rig_GALS.dpf', log=0)
        self.assertEqual(c['ligand_types']['value'], 'C FE')

    
    def test_lig_rig_flex(self):
        """set open ligand, set rigid filename and set flex filename
        """ 
        self.mv.AD41dpf_readFormattedLigand('./ind_zero.pdbqt', log=0)
        self.assertEqual(self.mv.DPF_LIGAND_TYPES, ['C'])
        self.mv.AD41dpf_readMacromolecule('./hsg1_rigid.pdbqt', log=0)
        self.mv.AD41dpf_readFlexRes('./hsg1_FE.pdbqt', log=0)
        self.assertEqual(self.mv.DPF_FLEXRES_TYPES, ['FE'])
        self.mv.AD41dpf_writeGALS('./zero_rig_FE_GALS.dpf', log=0)
        c=self.mv.dpo
        self.assertEqual(c['ligand_types']['value'], 'C FE')

    
    def test_lig_flex_rig(self):
        """set open ligand, set flex filename and set rigid filename
        """ 
        self.mv.AD41dpf_readFormattedLigand('./ind_zero.pdbqt', log=0)
        self.assertEqual(self.mv.DPF_LIGAND_TYPES, ['C'])
        self.mv.AD41dpf_readFlexRes('./hsg1_FE.pdbqt', log=0)
        self.assertEqual(self.mv.DPF_FLEXRES_TYPES, ['FE'])
        self.mv.AD41dpf_readMacromolecule('./hsg1_rigid.pdbqt', log=0)
        self.mv.AD41dpf_writeGALS('./zero_FE_rig_GALS.dpf', log=0)
        c=self.mv.dpo
        self.assertEqual(c['ligand_types']['value'], 'C FE')



class AD41dpf_choose_mol2_ligand_Tests(AD41dpf_BaseTests):

    def test_mol2_ligand_test(self):    
        """chooses ligand from mol2 file; checks written dpf
        """
        filename = "zma_test.dpf"
        if os.path.exists(filename):
            cmd = 'rm -f ' + filename
            os.system(cmd)
            print "removed dpf:", filename
        ligfilename = "1xyz_lig.pdbqt"
        if os.path.exists(ligfilename):
            cmd = 'rm -f ' + ligfilename
            os.system(cmd)
            print "removed ligfile: ", ligfilename

        self.mv.readMolecule('zma.mol2')
        self.mv.AD41tors_chooseLigand("zma")
        self.mv.AD41tors_autoRoot()
        self.mv.AD41tors_writeFormattedPDBQT('1xyz_lig.pdbqt')
        self.assertEqual(os.path.exists(ligfilename),True)
        self.mv.AD41dpf_readMacromolecule('./1xyz_rec.pdbqt')
        self.mv.AD41dpf_chooseFormattedLigand("zma", 1)
        self.mv.AD41dpf_writeGALS(filename)
        self.assertEqual(os.path.exists(filename),True)


class AD41dpf_ConfigBaseTests(unittest.TestCase):
    """
    setUp + tearDown form a fixture: working environment for the testing code
    """
    
        
    def startViewer(self):
        global mv
        #print 'in AD41dpf_ConfigBaseTests startViewer'
        if mv is None:
            from MolKit import Read
            import Tkinter
            from Pmv.moleculeViewer import MoleculeViewer
            mv = MoleculeViewer(trapExceptions=False)
            mv.loadModule('deleteCommands', 'Pmv')
            mv.loadModule('selectionCommands', 'Pmv')
            #change warningMsg format
            mv.setUserPreference(('warningMsgFormat', 'printed'), log = 0)
            mv.GUI.toolbarCheckbuttons['ADT']['Variable'].set(1)
            mv.Add_ADT()
            mv.loadModule('autodpfCommands', 'AutoDockTools')
            from AutoDockTools import setADTmode
            setADTmode("AD4.2", mv)
        self.mv = mv


    def setUp(self):
        """
        clean-up
        """
        if not hasattr(self, 'mv'):
            self.startViewer()


    def tearDown(self):
        """
        clean-up
        """
        #print 'in gpf tearDown'
        global ct, totalConfigCt
        #remove vinaDict
        try:
            delattr(self.mv, 'vinaDict')
        except:
            pass
        from AutoDockTools.DockingParameters import DockingParameters
        self.mv.dpo = DockingParameters()
        self.mv.dpo.vf = self.mv
        #reset gpo
        from AutoDockTools.GridParameters import GridParameters
        self.mv.gpo = GridParameters()
        self.mv.gpo.vf=self.mv
        #delete any molecules left due to errors
        for m in self.mv.Mols:
            self.mv.deleteMol(m)
        ct = ct + 1
        #print 'ct =', ct
        if ct==totalCt:
            print 'destroying mv'
            self.mv.Exit(0)
            del self.mv            
    


class AD41dpf_VinaConfigFileTests(AD41dpf_ConfigBaseTests):
    
    def test_vanilla_config(self):
        """write vanilla config file:
               load ligand + receptor 
               set search space
               write config file
               check whether new config file exists
        """ 
        filename = 'vanilla_config.txt'
        if os.path.exists(filename):
            cmd = "rm " + filename
            os.system(cmd)
        receptorfilename= 'hsg1.pdbqt'
        self.mv.AD41gpf_readMacromolecule(receptorfilename,preserve_input_charges=1)
        ligandfilename= 'ind.pdbqt'
        ligand_in_viewer = False
        for m in self.mv.Mols:
            if m.parser.filename==ligandfilename:
                self.mv.AD41gpf_chooseFormattedLigand('ind', 1, log=0)
                ligand_in_viewer = True
        if not ligand_in_viewer: self.mv.AD41gpf_readFormattedLigand(ligandfilename)
        self.mv.ADgpf_setGpo(gridcenterAuto=0, gridcenter=[2.5, 6.5, -7.5],npts=[50,50,50])
        self.mv.AD41dpf_readMacromolecule('./hsg1.pdbqt', log=0)
        self.mv.AD41dpf_writeCONFIG(filename, center_z='-7.5', 
                                     center_x='2.5', center_y='6.5', size_x='18.75', 
                                     size_y='18.75', size_z='18.75', receptor='hsg1.pdbqt', 
                                     out='ind_out.pdbqt', ligand='ind.pdbqt')
        self.assertEqual(os.path.exists(filename), True)
        for m in self.mv.Mols: self.mv.deleteMol(m)


    def test_vanilla_config_content(self):
        """write vanilla config file:
               load ligand + receptor 
               set search space
               write config file
               check whether new config file matches reference
        """ 
        filename = 'vanilla_config.txt'
        receptorfilename= 'hsg1.pdbqt'
        if os.path.exists(filename):
            cmd = "rm " + filename
            os.system(cmd)
        self.mv.AD41gpf_readMacromolecule(receptorfilename,preserve_input_charges=1)
        ligandfilename= 'ind.pdbqt'
        ligand_in_viewer = False
        for m in self.mv.Mols:
            if m.parser.filename==ligandfilename:
                self.mv.AD41gpf_chooseFormattedLigand('ind', 1, log=0)
                ligand_in_viewer = True
        if not ligand_in_viewer: self.mv.AD41gpf_readFormattedLigand(ligandfilename)
        self.mv.ADgpf_setGpo(gridcenterAuto=0, gridcenter=[2.5, 6.5, -7.5],npts=[50,50,50])
        self.mv.AD41dpf_readMacromolecule('./hsg1.pdbqt', log=0)
        self.mv.AD41dpf_writeCONFIG(filename, center_z='-7.5', 
                                     center_x='2.5', center_y='6.5', size_x='18.75', 
                                     size_y='18.75', size_z='18.75', receptor='hsg1.pdbqt', 
                                     out='ind_out.pdbqt', ligand='ind.pdbqt')
        self.assertEqual(os.path.exists(filename), True)
        testptr = open(filename)
        testlines = testptr.readlines()
        reffilename = 'ref_vanilla_config.txt'
        refptr = open(reffilename)
        reflines = refptr.readlines()
        for tline, rline in zip(testlines, reflines):
            self.assertEqual(tline, rline)
        for m in self.mv.Mols: self.mv.deleteMol(m)


    def test_score_only_config(self):
        """write score_only config file:
               load ligand + receptor 
               set search space
               write score_only config file
               check whether new config file matches reference
        """ 
        filename = 'config_score_only.txt'
        if os.path.exists(filename):
            cmd = "rm " + filename
            os.system(cmd)
        receptorfilename= 'hsg1.pdbqt'
        if 'hsg1' not in self.mv.Mols.name:
            self.mv.AD41gpf_readMacromolecule(receptorfilename,preserve_input_charges=1)
        ligandfilename= 'ind.pdbqt'
        ligand_in_viewer = False
        for m in self.mv.Mols:
            if m.parser.filename==ligandfilename:
                self.mv.AD41gpf_chooseFormattedLigand('ind', log=0)
                ligand_in_viewer = True
        if not ligand_in_viewer: self.mv.AD41gpf_readFormattedLigand(ligandfilename)
        self.mv.ADgpf_setGpo(gridcenterAuto=0, gridcenter=[2.5, 6.5, -7.5],npts=[50,50,50])
        self.mv.AD41dpf_readMacromolecule('./hsg1.pdbqt', log=0)
        self.mv.AD41dpf_writeCONFIG(filename, receptor='hsg1.pdbqt', ligand='ind.pdbqt',score_only=1)
        self.assertEqual(os.path.exists(filename), True)
        testptr = open(filename)
        testlines = testptr.readlines()
        reffilename = 'ref_config_score_only.txt'
        refptr = open(reffilename)
        reflines = refptr.readlines()
        for tline, rline in zip(testlines, reflines):
            self.assertEqual(tline, rline)
        for m in self.mv.Mols: self.mv.deleteMol(m)


    def test_local_only_config(self):
        """write local_only config file:
               load ligand + receptor 
               set search space
               write local_only config file
               check whether new config file matches reference
        """ 
        filename = 'config_local_only.txt'
        if os.path.exists(filename):
            cmd = "rm " + filename
            os.system(cmd)
        receptorfilename= 'hsg1.pdbqt'
        if 'hsg1' not in self.mv.Mols.name:
            self.mv.AD41gpf_readMacromolecule(receptorfilename,preserve_input_charges=1)
        ligandfilename= 'ind.pdbqt'
        ligand_in_viewer = False
        for m in self.mv.Mols:
            if m.parser.filename==ligandfilename:
                self.mv.AD41gpf_chooseFormattedLigand('ind', log=0)
                ligand_in_viewer = True
        if not ligand_in_viewer: self.mv.AD41gpf_readFormattedLigand(ligandfilename)
        self.mv.ADgpf_setGpo(gridcenterAuto=0, gridcenter=[2.5, 6.5, -7.5],npts=[50,50,50])
        self.mv.AD41dpf_readMacromolecule('./hsg1.pdbqt', log=0)
        self.mv.AD41dpf_writeCONFIG(filename, center_z='-7.5', 
                                     center_x='2.5', center_y='6.5', size_x='18.75', 
                                     size_y='18.75', size_z='18.75', receptor='hsg1.pdbqt', 
                                     out='ind_local_only.pdbqt', ligand='ind.pdbqt',
                                     local_only=True)
        self.assertEqual(os.path.exists(filename), True)
        testptr = open(filename)
        testlines = testptr.readlines()
        reffilename = 'ref_config_local_only.txt'
        refptr = open(reffilename)
        reflines = refptr.readlines()
        for tline, rline in zip(testlines, reflines):
            self.assertEqual(tline, rline)
        for m in self.mv.Mols: self.mv.deleteMol(m)


if __name__ == '__main__':
    test_cases = [
        'AD41dpf_BaseTests',
        'AD41dpf_readDPFTests',
        'AD41dpf_flexResTests',
        'AD41dpf_formatting_flexResTests',
        'AD41dpf_FlexRes_Lig_Order_Tests',
        'AD41dpf_choose_mol2_ligand_Tests',
        'AD41dpf_VinaConfigFileTests',
            ]
    unittest.main( argv=([__name__ ,] + test_cases) )






#
#
#  $Id: test_PyAutoDock.py,v 1.6 2007/09/27 19:05:18 annao Exp $ 
#  $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Tests/test_PyAutoDock.py,v 1.6 2007/09/27 19:05:18 annao Exp $
#
#


import unittest, glob, os
import numpy.oldnumeric as Numeric
from string import find, split
from MolKit import molecule, protein
#molecule.bhtreeFlag = 0
#protein.bhtreeFlag = 0

mv = None
ct = 0
totalCt = 10
class PyAutoDock_Tests(unittest.TestCase):
    """
    setUp + tearDown form a fixture: working environment for the testing code
    """

    lines = [
    "at#  elem       estat       hbond         vdw         desolv",
    "  1    N      -0.0099      -0.0001      -0.2204       0.0000",
    "  2    C      -0.0481       0.0000      -0.2896      -0.0976",
    "  3    C      -0.0540       0.0000      -0.2754      -0.0983",
    "  4    C      -0.0385       0.0000      -0.3023      -0.1049",
    "  5    O      -0.0026      -0.0014      -0.2562       0.2360",
    "  6    N       0.0535      -0.0008      -0.2344       0.0000",
    "  7    N       0.1733      -0.0001      -0.2173       0.0000",
    "  8    C      -0.0742       0.0000      -0.3414      -0.1175",
    "  9    C      -0.0517       0.0000      -0.3405      -0.1182",
    " 10    C      -0.1279       0.0000      -0.4233      -0.1189",
    " 11    C      -0.1292       0.0000      -0.4071      -0.1187",
    " 12    C      -0.0325       0.0000      -0.3249      -0.1129",
    " 13    C      -0.0429       0.0000      -0.3252      -0.1121",
    " 14    C      -0.0125       0.0000      -0.3426      -0.1163",
    " 15    C      -0.0638       0.0000      -0.2889      -0.1015",
    " 16    O       0.0047      -0.0029      -0.2253       0.2360",
    " 17    N       0.1447      -0.0009      -0.2360       0.0000",
    " 18    C      -0.0283       0.0000      -0.3450      -0.1054",
    " 19    C      -0.0295       0.0000      -0.4043      -0.1014",
    " 20    C      -0.0122       0.0000      -0.4316      -0.1092",
    " 21    A       0.0028       0.0000      -0.4923      -0.0194",
    " 22    A      -0.0010       0.0000      -0.5748      -0.0230",
    " 23    A      -0.0002       0.0000      -0.5267      -0.0254",
    " 24    A      -0.0001       0.0000      -0.4778      -0.0237",
    " 25    A      -0.0011       0.0000      -0.3777      -0.0203",
    " 26    A       0.0021       0.0000      -0.3725      -0.0180",
    " 27    H      -0.0246      -0.0033      -0.0199       0.1180",
    " 28    H      -0.1724      -0.2910      -0.0330       0.1180",
    " 29    O       0.7385      -0.0006      -0.4405       0.2360",
    " 30    H      -0.5705      -0.4923      -0.0176       0.1180",
    " 31    A       0.0077       0.0000      -0.3121      -0.0168",
    " 32    A      -0.0012       0.0000      -0.3468      -0.0177",
    " 33    A      -0.0001       0.0000      -0.3358      -0.0160",
    " 34    A       0.0001       0.0000      -0.3722      -0.0165",
    " 35    A       0.0000       0.0000      -0.2975      -0.0134",
    " 36    A       0.0000       0.0000      -0.3137      -0.0139",
    " 37    C      -0.0033       0.0000      -0.3657      -0.1035",
    " 38    A      -0.0013       0.0000      -0.2973      -0.0132",
    " 39    A       0.0462       0.0000      -0.3079      -0.0125",
    " 40    A       0.0007       0.0000      -0.2723      -0.0126",
    " 41    N      -0.2328      -0.3911      -0.1800       0.0000",
    " 42    A       0.0460       0.0000      -0.1824      -0.0076",
    " 43    A       0.0032       0.0000      -0.1860      -0.0090",
    " 44    C      -0.0017       0.0000      -0.3774      -0.1296",
    " 45    C      -0.0039       0.0000      -0.4594      -0.1376",
    " 46    C      -0.0036       0.0000      -0.4036      -0.1370",
    " 47    C       0.0001       0.0000      -0.4630      -0.1401",
    " 48    O       0.0935      -0.2694      -0.4128       0.2360",
    " 49    H      -0.0474      -0.4145      -0.0474       0.1180",
    ]

    estat_weight = 0.1146 # Autogrid3 weight
    dsolv_weight = 0.1711 # Autogrid3 weight
    hbond_weight = 0.0656 # Autogrid3 weight
    vdw_weight = 0.1485 # Autogrid3 weight
    #data_file = open('evaluator_results')
    #lines = data_file.readlines()
    #data_file.close()
    estat_data = Numeric.array([float(split(l)[2]) for l in lines[1:]])
    hbond_data = Numeric.array([float(split(l)[3]) for l in lines[1:]])
    vdw_data   = Numeric.array([float(split(l)[4]) for l in lines[1:]])
    dsolv_data = Numeric.array([float(split(l)[5]) for l in lines[1:]])
    ad305_data= estat_data+hbond_data+vdw_data+dsolv_data
        
    def startViewer(self):
        global mv
        #print 'in test_AD startViewer'
        if mv is None:
            from MolKit import Read
            import Tkinter
            from Pmv.moleculeViewer import MoleculeViewer
            mv = MoleculeViewer(withShell=False)
            #mv = MoleculeViewer(trapExceptions = False)
            mv.loadModule('pyAutoDockCommands', 'AutoDockTools')
            mv.loadModule('deleteCommands', 'Pmv')
            mv.loadModule('selectionCommands', 'Pmv')
            #change warningMsg format
            mv.setUserPreference(('warningMsgFormat', 'printed'), log = 0)
        self.mv = mv


    def setUp(self):
        """
        clean-up
        """
        if not hasattr(self, 'mv'):
            self.startViewer()
        if "pytest_hsg1" not in self.mv.Mols.name:
            mv.readMolecule("pytest_hsg1.pdbqs")
        if "docked" not in self.mv.Mols.name:
            mv.readMolecule("docked.pdbqs")
        self.ligAts = mv.Mols.get('docked')[0].allAtoms
        self.lenLig = len(self.ligAts)


    def tearDown(self):
        """
        clean-up
        """
        global ct, totalCt
        ct = ct + 1
        #print 'ct =', ct
        if ct==totalCt:
            print 'destroying mv'
            self.mv.Exit(0)
            del self.mv


    def test_VDW_pyAutoDock_vs_evaluator(self):
        """ compare AD3 vdw energies in ADT with stored values from PyAutoDock"""
        self.mv.PyAD_calcVDWEnergies("pytest_hsg1:::", "docked:::", self.vdw_weight)
        for i, a in zip(range(self.lenLig), self.ligAts):
            self.assertAlmostEqual(a.vdw_energy, self.vdw_data[i], 2)


    def test_HB_pyAutoDock_vs_evaluator(self):
        """ compare AD3 hbond energies in ADT with stored values from PyAutoDock"""
        self.mv.PyAD_calcHBONDEnergies("pytest_hsg1:::", "docked:::", self.hbond_weight)
        for i, a in zip(range(self.lenLig), self.ligAts):
            self.assertAlmostEqual(a.hb_energy, self.hbond_data[i], 1)


    def test_DSOLV_pyAutoDock_vs_evaluator(self):
        """ compare AD3 dsolv energies in ADT with stored values from PyAutoDock"""
        self.mv.PyAD_calcDSOLVEnergies("pytest_hsg1:::", "docked:::", self.dsolv_weight)
        for i, a in zip(range(self.lenLig), self.ligAts):
            self.assertAlmostEqual(a.dsolv_energy, self.dsolv_data[i], 3)


    def test_ESTAT_pyAutoDock_vs_evaluator(self):
        """ compare AD3 estat energies in ADT with stored values from PyAutoDock"""
        self.mv.PyAD_calcESTATEnergies("pytest_hsg1:::", "docked:::", self.estat_weight)
        for i, a in zip(range(self.lenLig), self.ligAts):
            #print i, ':', a.name, ':', a.estat_energy, ':', self.estat_data[i]
            self.assertAlmostEqual(a.estat_energy, self.estat_data[i], 2)


    def test_AD305_pyAutoDock_vs_evaluator(self):
        """ compare sum of AD305 energies computed in ADT with stored values from PyAutoDock
        """
        self.mv.PyAD_calcAD3Energies("pytest_hsg1:::", "docked:::")
        max_err = -20
        max_pErr = -20
        for i, a in zip(range(self.lenLig), self.ligAts):
            #apparently oxygen 29 required this slop in value comparison
            #print i, ':', a.name, ':', a.ad305_energy, ':', self.ad305_data[i]
            self.assertAlmostEqual(a.ad305_energy, self.ad305_data[i], 1)
            err = abs(a.ad305_energy-self.ad305_data[i])
            if err>max_err:
                #print "resetting err for ", a.name, ':',  err
                max_err = err
            self.assertEqual(err < .02, True)
            percent_err = abs(err/a.ad305_energy)*100.
            if percent_err>max_pErr:
                #print "reseting for ", a.name, ':', percent_err
                max_pErr= percent_err
            self.assertEqual(percent_err < 10, True)
        #print "max_pErr=", max_pErr
        #print "max_err=", max_err
            


class PyAutoDock4_Tests(unittest.TestCase):
    """
    setUp + tearDown form a fixture: working environment for the testing code
    """
    lines = [
        "at#  at_name   estat     hbond       vdw    dsolvLig  dsolvRec",
        "  1    N1      -0.0153    0.0000   -0.2341   -0.0270    0.0375",
        "  2    C1      -0.0697    0.0000   -0.3043    0.0873    0.0664",
        "  3    C2      -0.0816    0.0000   -0.2893    0.0343    0.0704",
        "  4    C3      -0.0697    0.0000   -0.3176    0.0588    0.0610",
        "  5    O1       0.0352   -0.0021   -0.2691    0.0224    0.0289",
        "  6    N2       0.0946    0.0000   -0.2501    0.1109    0.0424",
        "  7    N3       0.2671    0.0000   -0.2303    0.1297    0.0459",
        "  8    C8      -0.1093    0.0000   -0.3587    0.0103    0.0620",
        "  9    C9      -0.0816    0.0000   -0.3584    0.0935    0.0486",
        " 10    C10     -0.1956    0.0000   -0.4447    0.0110    0.0874",
        " 11    C11     -0.1998    0.0000   -0.4278    0.0034    0.0894",
        " 12    C12     -0.0505    0.0000   -0.3413   -0.0461    0.0703",
        " 13    C13     -0.0633    0.0000   -0.3416   -0.0257    0.0683",
        " 14    C14     -0.0206    0.0000   -0.3599   -0.0449    0.0539",
        " 15    C21     -0.0999    0.0000   -0.3035    0.0465    0.0712",
        " 16    O3       0.0451   -0.0044   -0.2367    0.0225    0.0313",
        " 17    N4       0.1862    0.0000   -0.2527    0.1056    0.0595",
        " 18    C22     -0.0450    0.0000   -0.3624    0.0088    0.0883",
        " 19    C23     -0.0558    0.0000   -0.4247    0.0096    0.1116",
        " 20    C24     -0.0265    0.0000   -0.4534   -0.0316    0.1099",
        " 21    A25      0.0101    0.0000   -0.5171   -0.0003    0.0970",
        " 22    A26     -0.0019    0.0000   -0.6038   -0.0297    0.0902",
        " 23    A27     -0.0002    0.0000   -0.5533   -0.0385    0.0794",
        " 24    A28     -0.0002    0.0000   -0.5019   -0.0360    0.0648",
        " 25    A29     -0.0024    0.0000   -0.3967   -0.0250    0.0676",
        " 26    A30      0.0058    0.0000   -0.3913   -0.0122    0.0813",
        " 27    H4      -0.0465   -0.0018   -0.0250    0.1140    0.0000",
        " 28    H32     -0.1729   -0.4870   -0.0441    0.1198    0.0000",
        " 29    O2       1.1325   -0.0008   -0.4627    0.1187    0.0651",
        " 30    H21     -0.8732   -0.4870   -0.0280    0.2014    0.0000",
        " 31    A15      0.0120    0.0000   -0.3279    0.0059    0.0603",
        " 32    A16     -0.0014    0.0000   -0.3644   -0.0235    0.0729",
        " 33    A20     -0.0004    0.0000   -0.3528   -0.0216    0.0618",
        " 34    A17      0.0002    0.0000   -0.3910   -0.0252    0.0812",
        " 35    A18      0.0000    0.0000   -0.3125   -0.0211    0.0785",
        " 36    A19      0.0001    0.0000   -0.3295   -0.0216    0.0676",
        " 37    C31     -0.0039    0.0000   -0.3848    0.0940    0.0546",
        " 38    A32     -0.0014    0.0000   -0.3125   -0.0153    0.0564",
        " 39    A33      0.0451    0.0000   -0.3235    0.0321    0.0628",
        " 40    A36      0.0009    0.0000   -0.2862   -0.0154    0.0519",
        " 41    N5      -0.2140   -0.3009   -0.1891    0.0405    0.0423",
        " 42    A34      0.0419    0.0000   -0.1916    0.0179    0.0514",
        " 43    A35      0.0031    0.0000   -0.1954   -0.0087    0.0434",
        " 44    C4      -0.0037    0.0000   -0.3965   -0.0698    0.0509",
        " 45    C5      -0.0074    0.0000   -0.4826   -0.0636    0.0634",
        " 46    C6      -0.0078    0.0000   -0.4240   -0.0625    0.0440",
        " 47    C7      -0.0026    0.0000   -0.4864   -0.0644    0.0383",
        " 48    O4       0.2186   -0.3993   -0.4336    0.1001    0.0710",
        " 49    H35     -0.1014   -0.4421   -0.0696    0.1722    0.0000",
    ]

    vdw_weight4 = 0.1560 # Autogrid4 weight
    hbond_weight4 = 0.0974 # Autogrid4 weight
    estat_weight4 = 0.1465 # Autogrid4 weight
    dsolv_weight4 = 0.1159 # Autogrid4 weight
    #data_file4 = open('evaluator4_results')
    #lines = data_file4.readlines()
    #data_file4.close()
    estat_data4 = Numeric.array([float(split(l)[2]) for l in lines[1:]])
    hbond_data4 = Numeric.array([float(split(l)[3]) for l in lines[1:]])
    vdw_data4   = Numeric.array([float(split(l)[4]) for l in lines[1:]])
    dsolv_data4 = Numeric.array([(float(split(l)[5])+float(split(l)[6])) for l in lines[1:]])
    ad4_data= estat_data4+hbond_data4+vdw_data4+dsolv_data4
        

    def startViewer(self):
        global mv
        #print 'in test_AD startViewer'
        if mv is None:
            from MolKit import Read
            import Tkinter
            from Pmv.moleculeViewer import MoleculeViewer
            mv = MoleculeViewer(withShell=False)
            #mv = MoleculeViewer(trapExceptions = False)
            mv.loadModule('pyAutoDockCommands', 'AutoDockTools')
            mv.loadModule('deleteCommands', 'Pmv')
            mv.loadModule('selectionCommands', 'Pmv')
            #change warningMsg format
            mv.setUserPreference(('warningMsgFormat', 'printed'), log = 0)
            mv.readMolecule("hsg1.pdbqt")
            mv.readMolecule("docked4.pdbqt")
        self.mv = mv


    def setUp(self):
        """
        clean-up
        """
        if not hasattr(self, 'mv'):
            self.startViewer()
        if "hsg1" not in self.mv.Mols.name:
            mv.readMolecule("hsg1.pdbqt")
        if "docked4" not in self.mv.Mols.name:
            mv.readMolecule("docked4.pdbqt")
        self.ligAts = mv.Mols.get("docked4")[0].allAtoms
        self.lenLig = len(self.ligAts)


    def tearDown(self):
        """
        clean-up
        """
        global ct, totalCt
        ct = ct + 1
        #print 'ct =', ct
        if ct==totalCt:
            print 'destroying mv'
            self.mv.Exit(0)
            del self.mv


    def test_VDW_pyAutoDock4_vs_evaluator4(self):
        """ compare vdw4 energies in ADT with stored values from PyAutoDock"""
        self.mv.PyAD4_calcVDWEnergies("hsg1:::", "docked4:::", self.vdw_weight4)
        max_err = -20
        max_pe = 0.
        max_pe2 = 0.
        for i, a in zip(range(self.lenLig), self.ligAts):
            eval_data = self.vdw_data4[i]
            adt_data = a.ad4_vdw_energy
            #print i, ':', a.name, ':',adt_data, ':', eval_data,
            self.assertAlmostEqual(adt_data, eval_data, 1)
            err = abs(adt_data-eval_data)
            #print "err=", err
            if err>max_err:
                max_err = err
                #print "max_err set to ", max_err, ' for ',  a.name
            self.assertEqual(err < .020, True)
            percent_err = abs(err/eval_data)*100.
            if percent_err>max_pe:
                max_pe=percent_err
            #print "percent_err=", percent_err, ':', abs(adt_data)>.075
            if abs(adt_data)>.06:
                if percent_err>max_pe2:
                    max_pe2=percent_err
                self.assertEqual(percent_err < 3, True)
        #print "max_pe=", max_pe
        #print "max_pe2=", max_pe2


    def test_HB_pyAutoDock4_vs_evaluator4(self):
        """ compare hbond energies4  in ADT with stored values from PyAutoDock"""
        self.mv.PyAD4_calcHBONDEnergies("hsg1:::", "docked4:::", self.hbond_weight4)
        for i, a in zip(range(self.lenLig), self.ligAts):
            adt_energy = a.ad4_hb_energy
            #adt_sum_energy = a.sum_hb_energy
            eval_energy = self.hbond_data4[i]
            #print i, ':', a.name,':', adt_energy, '-', eval_energy
            #self.assertAlmostEqual(a.ad4_hb_energy, self.hbond_data4[i], 1)
            err = abs(adt_energy - eval_energy)
            #print "err=", err
            self.assertEqual(err < .10, True)
            #if a.ad4_hb_energy!=0.0:
            if adt_energy!=0.0:
                percent_err = abs(err/adt_energy)*100.
                #print "percent_err=", percent_err
                if abs(adt_energy)>.075:
                    self.assertEqual(percent_err < 30, True)


    def test_DSOLV_pyAutoDock4_vs_evaluator4(self):
        """ compare dsolv energies in ADT4 with stored values from PyAutoDock"""
        self.mv.PyAD4_calcDSOLVEnergies("hsg1:::", "docked4:::", self.dsolv_weight4)
        for i, a in zip(range(self.lenLig), self.ligAts):
            #print i, ':', a.name, ':', a.ad4_dsolv_energy, ':', self.dsolv_data4[i]
            self.assertAlmostEqual(a.ad4_dsolv_energy, self.dsolv_data4[i], 3)


    def test_ESTAT_pyAutoDock4_vs_evaluator4(self):
        """ compare estat energies in ADT with stored values from PyAutoDock"""
        self.mv.PyAD4_calcESTATEnergies("hsg1:::", "docked4:::", self.estat_weight4)
        max_err = -20
        for i, a in zip(range(self.lenLig), self.ligAts):
            err = abs(a.ad4_estat_energy-self.estat_data4[i])
            #print i, ':', a.name, ':', a.ad4_estat_energy, ':', self.estat_data4[i]
            if err>max_err:
                max_err = err
                ##print "max_err set to ", max_err, ' for ',  a.name
            self.assertAlmostEqual(a.ad4_estat_energy, self.estat_data4[i], 2)
        ##print 'max_err=', max_err


    def test_AD4_pyAutoDock4_vs_evaluator4(self):
        """ compare sum of AD4 energies computed in ADT with stored values from PyAutoDock
        """
        self.mv.PyAD4_calcAD4Energies("hsg1:::", "docked4:::")
        max_err = -20
        max_Perr = -20
        for i, a in zip(range(self.lenLig), self.ligAts):
            #apparently oxygen 29 required this slop in value comparison
            #print i, ':', a.name, ':', a.ad4_energy, ':', self.ad4_data[i]
            #print round(a.ad4_energy,1)==round(self.ad4_data[i],1)
            #self.assertAlmostEqual(a.ad4_energy, self.ad4_data[i], 1)
            err = abs(a.ad4_energy-self.ad4_data[i])
            #print "err=", err
            if err>max_err:
                max_err = err
                #print "reset max_err for ", a.name
            self.assertEqual(err < .3, True)
            percent_err = abs(err/a.ad4_energy)*100.
            if a.ad4_energy>.05:
                if percent_err>max_Perr:
                    max_Perr = percent_err
                    #print ">>> reset max_Perr for ", a.name, " percent_err=", percent_err
                #print 'percent_err=', percent_err
                self.assertEqual(percent_err < 5, True)
        #print "max_err=", max_err
        #print "max_Perr=", max_Perr
            


if __name__ == '__main__':
    test_cases = [
        'PyAutoDock_Tests',
        'PyAutoDock4_Tests',
        #'PyAutoDock_CalcClosestPairs',
            ]
    unittest.main( argv=([__name__ ,] + test_cases) )
    #unittest.main( argv=([__name__ , '-v'] + test_cases) )








        

   





